@@ -5,15 +5,14 @@ Fine grained Plotting Features of Map
 
 How to control various plotting features of map.
 """
+import matplotlib.pyplot as plt
+
 import astropy.units as u
 from astropy.coordinates import SkyCoord
-import matplotlib.pyplot as plt
-from matplotlib.colors import LogNorm
 
 import sunpy.map
 from sunpy.data.sample import AIA_171_IMAGE
 
-
 ###############################################################################
 # SkyCoord module provides flexible celestial coordinate representation and a
 # draw_limb method draws an edge as seen against the dark sky background. Date of the
@@ -22,7 +21,7 @@ aiamap = sunpy.map.Map(AIA_171_IMAGE)
 
 bottom_left = SkyCoord(-400*u.arcsec, -900*u.arcsec, frame=aiamap.coordinate_frame)
 top_right = SkyCoord(800*u.arcsec, 700*u.arcsec, frame=aiamap.coordinate_frame)
-aiamap_sub = aiamap.submap(bottom_left, top_right)
+aiamap_sub = aiamap.submap(bottom_left, top_right=top_right)
 
 title_obsdate = aiamap_sub.date.strftime('%Y-%b-%d %H:%M:%S')
 
@@ -56,7 +55,7 @@ lon.coord_wrap = 180
 lon.set_major_formatter('dd')
 
 # Plot the Heliographic Stonyhurst grid
-overlay.grid(color='blue', linewidth=2, linestyle='dashed')
+overlay.grid(color='tab:blue', linewidth=2, linestyle='dashed')
 # Switch off the helioprojective grid
 ax.grid(False)
 
