@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
 import datetime
 
 import astropy.units as u
+from astropy.coordinates import BaseCoordinateFrame, CoordinateAttribute, SkyCoord, TimeAttribute
 from astropy.time import Time
-from astropy.coordinates import TimeAttribute, CoordinateAttribute
 
 from sunpy.time import parse_time
 
@@ -103,6 +102,10 @@ class ObserverCoordinateAttribute(CoordinateAttribute):
         if isinstance(value, str):
             return value, False
         else:
+            # Upgrade the coordinate to a `SkyCoord` so that frame attributes will be merged
+            if isinstance(value, BaseCoordinateFrame) and not isinstance(value, self._frame):
+                value = SkyCoord(value)
+
             return super().convert_input(value)
 
     def _convert_string_to_coord(self, out, obstime):
@@ -112,7 +115,6 @@ class ObserverCoordinateAttribute(CoordinateAttribute):
         """
 
         # Import here to prevent circular import
-        from .frames import HeliographicStonyhurst
         from .ephemeris import get_body_heliographic_stonyhurst
 
         obscoord = get_body_heliographic_stonyhurst(out, obstime)
@@ -127,11 +129,8 @@ class ObserverCoordinateAttribute(CoordinateAttribute):
     def __get__(self, instance, frame_cls=None):
         # If instance is None then we can't get obstime so it doesn't matter.
         if instance is not None:
-            # Get observer if the instance has one, or the default.
-            observer = getattr(instance, '_' + self.name, self.default)
-
-            # We have an instance of a frame, so get obstime
-            obstime = getattr(instance, 'obstime', None)
+            observer = getattr(instance, '_' + self.name)
+            obstime = getattr(instance, 'obstime', None)  # TODO: Why is this `None` needed?
 
             # If the observer is a string and we have obstime then calculate
             # the position of the observer.
