@@ -9,19 +9,25 @@ import numpy as np
 import astropy.units as u
 from astropy.coordinates import Attribute, ConvertError
 from astropy.coordinates.baseframe import BaseCoordinateFrame, RepresentationMapping
-from astropy.coordinates.representation import (CartesianRepresentation, SphericalRepresentation,
-                                                CylindricalRepresentation,
-                                                UnitSphericalRepresentation)
+from astropy.coordinates.representation import (
+    CartesianRepresentation,
+    CylindricalRepresentation,
+    SphericalRepresentation,
+    UnitSphericalRepresentation,
+)
+from astropy.time import Time
 
 from sunpy.sun.constants import radius as _RSUN
-
-from .frameattributes import TimeFrameAttributeSunPy, ObserverCoordinateAttribute
-
-from sunpy.util.decorators import add_common_docstring
 from sunpy.time.time import _variables_for_parse_time_docstring
+from sunpy.util.decorators import add_common_docstring, deprecated
+from .frameattributes import ObserverCoordinateAttribute, TimeFrameAttributeSunPy
+
+_J2000 = Time('J2000.0', scale='tt')
 
 __all__ = ['HeliographicStonyhurst', 'HeliographicCarrington',
-           'Heliocentric', 'Helioprojective']
+           'Heliocentric', 'Helioprojective',
+           'HeliocentricEarthEcliptic', 'GeocentricSolarEcliptic',
+           'HeliocentricInertial', 'GeocentricEarthEquatorial']
 
 
 def _frame_parameters():
@@ -58,6 +64,12 @@ def _frame_parameters():
     ret['radius'] = ("radius : `~astropy.units.Quantity`, optional\n"
                      "        The radial distance coordinate from Sun center for this object.\n"
                      "        Defaults to the radius of the Sun. Not needed if ``data`` is given.")
+    ret['distance_sun'] = ("distance : `~astropy.units.Quantity`, optional\n"
+                           "        The distance coordinate from Sun center for this object.\n"
+                           "        Not needed if ``data`` is given.")
+    ret['distance_earth'] = ("distance : `~astropy.units.Quantity`, optional\n"
+                             "        The distance coordinate from Earth center for this object.\n"
+                             "        Not needed if ``data`` is given.")
     ret['xyz'] = ("x : `~astropy.units.Quantity`, optional\n"
                   "        X-axis coordinate for this object. Not needed if ``data`` is given.\n"
                   "    y : `~astropy.units.Quantity`, optional\n"
@@ -69,6 +81,9 @@ def _frame_parameters():
                        "        it must be a solar system body that can be parsed by\n"
                        "        `~sunpy.coordinates.ephemeris.get_body_heliographic_stonyhurst`\n"
                        "        at the time ``obstime``. Defaults to Earth center.")
+    ret['equinox'] = (f"equinox : {_variables_for_parse_time_docstring()['parse_time_types']}\n"
+                      "        The date for the mean vernal equinox.\n"
+                      "        Defaults to the J2000.0 equinox.")
 
     return ret
 
@@ -109,6 +124,14 @@ class SunPyBaseCoordinateFrame(BaseCoordinateFrame):
             data.lon.wrap_angle = self._wrap_angle
         return data
 
+    @property
+    def size(self):
+        """
+        Returns the size of the underlying data if it exists, else returns 0.  This overrides the
+        property in `~astropy.coordinates.BaseCoordinateFrame`.
+        """
+        return self.data.size if self.has_data else 0
+
     def __str__(self):
         """
         We override this here so that when you print a SkyCoord it shows the
@@ -120,8 +143,46 @@ class SunPyBaseCoordinateFrame(BaseCoordinateFrame):
             return super().__str__()
 
 
+class BaseHeliographic(SunPyBaseCoordinateFrame):
+    """
+    Base class for HeliographicCarrington (HGC) and HeliographicStonyhurst (HGS) frames.
+
+    This class is not intended to be used directly and has no transformations defined.
+    """
+    default_representation = SphericalRepresentation
+
+    frame_specific_representation_info = {
+        SphericalRepresentation: [RepresentationMapping(reprname='lon',
+                                                        framename='lon',
+                                                        defaultunit=u.deg),
+                                  RepresentationMapping(reprname='lat',
+                                                        framename='lat',
+                                                        defaultunit=u.deg),
+                                  RepresentationMapping(reprname='distance',
+                                                        framename='radius',
+                                                        defaultunit=None)],
+        CartesianRepresentation: [RepresentationMapping(reprname='x',
+                                                        framename='x'),
+                                  RepresentationMapping(reprname='y',
+                                                        framename='y'),
+                                  RepresentationMapping(reprname='z',
+                                                        framename='z')]
+    }
+
+    def __init__(self, *args, **kwargs):
+        kwargs.get('representation_type', None)
+
+        super().__init__(*args, **kwargs)
+
+        # Make 3D if specified as 2D
+        if (self._data is not None and self._data.norm().unit is u.one
+                and u.allclose(self._data.norm(), 1*u.one)):
+
+            self._data *= _RSUN.to(u.km)
+
+
 @add_common_docstring(**_frame_parameters())
-class HeliographicStonyhurst(SunPyBaseCoordinateFrame):
+class HeliographicStonyhurst(BaseHeliographic):
     """
     A coordinate or frame in the Stonyhurst Heliographic (HGS) system.
 
@@ -134,12 +195,12 @@ class HeliographicStonyhurst(SunPyBaseCoordinateFrame):
     represented using Cartesian components.
 
     A new instance can be created using the following signatures
-    (note that ``obstime`` and ``representation_type`` must be supplied as
-    keywords)::
+    (note that if supplied, ``obstime`` and ``representation_type`` must be
+    keyword arguments)::
 
-        HeliographicStonyhurst(lon, lat, obstime)
-        HeliographicStonyhurst(lon, lat, radius, obstime)
-        HeliographicStonyhurst(x, y, z, obstime, representation_type='cartesian')
+        HeliographicStonyhurst(lon, lat, obstime=obstime)
+        HeliographicStonyhurst(lon, lat, radius, obstime=obstime)
+        HeliographicStonyhurst(x, y, z, representation_type='cartesian', obstime=obstime)
 
     Parameters
     ----------
@@ -179,51 +240,10 @@ class HeliographicStonyhurst(SunPyBaseCoordinateFrame):
     ``rsun``.
     """
     name = "heliographic_stonyhurst"
-    default_representation = SphericalRepresentation
-
-    frame_specific_representation_info = {
-        SphericalRepresentation: [RepresentationMapping(reprname='lon',
-                                                        framename='lon',
-                                                        defaultunit=u.deg),
-                                  RepresentationMapping(reprname='lat',
-                                                        framename='lat',
-                                                        defaultunit=u.deg),
-                                  RepresentationMapping(reprname='distance',
-                                                        framename='radius',
-                                                        defaultunit=None)],
-        CartesianRepresentation: [RepresentationMapping(reprname='x',
-                                                        framename='x'),
-                                  RepresentationMapping(reprname='y',
-                                                        framename='y'),
-                                  RepresentationMapping(reprname='z',
-                                                        framename='z')]
-    }
-
-    def __init__(self, *args, **kwargs):
-        _rep_kwarg = kwargs.get('representation_type', None)
-
-        super().__init__(*args, **kwargs)
-
-        # Make 3D if specified as 2D
-        # If representation was explicitly passed, do not change the rep.
-        if not _rep_kwarg:
-            if isinstance(self._data, UnitSphericalRepresentation):
-                self._data = self.spherical
-
-    def represent_as(self, base, s='base', in_frame_units=False):
-        """
-        Unless the requested representation is UnitSphericalRepresentation, scale a coordinate with
-        dimensionless length so that it has the length of the solar radius.
-        """
-        data = super().represent_as(base, s, in_frame_units=in_frame_units)
-        if not isinstance(data, UnitSphericalRepresentation) and \
-           data.norm().unit is u.one and u.allclose(data.norm(), 1*u.one):
-            data *= _RSUN.to(u.km)
-        return data
 
 
 @add_common_docstring(**_frame_parameters())
-class HeliographicCarrington(HeliographicStonyhurst):
+class HeliographicCarrington(BaseHeliographic):
     """
     A coordinate or frame in the Carrington Heliographic (HGC) system.
 
@@ -231,19 +251,23 @@ class HeliographicCarrington(HeliographicStonyhurst):
     - The Z-axis (+90 degrees latitude) is aligned with the Sun's north pole.
     - The X-axis and Y-axis rotate with a period of 25.38 days.
 
-    This system differs from Stonyhurst Heliographic (HGS) in its definition of longitude.
+    This system differs from Stonyhurst Heliographic (HGS) in its definition of longitude.  This
+    longitude is an "apparent" longitude because it takes into account the time it takes for light
+    to travel from the Sun's surface to the observer.  Thus, the observer needs to be specified to
+    be able to transform to any other coordinate frame.
 
     A new instance can be created using the following signatures
-    (note that ``obstime`` must be supplied as a keyword)::
+    (note that if supplied, ``obstime`` and ``observer`` must be a keyword argument)::
 
-        HeliographicCarrington(lon, lat, obstime)
-        HeliographicCarrington(lon, lat, radius, obstime)
+        HeliographicCarrington(lon, lat, obstime=obstime, observer=observer)
+        HeliographicCarrington(lon, lat, radius, obstime=obstime, observer=observer)
 
     Parameters
     ----------
     {data}
     {lonlat}
     {radius}
+    {observer}
     {common}
 
     Examples
@@ -253,27 +277,30 @@ class HeliographicCarrington(HeliographicStonyhurst):
     >>> import astropy.units as u
     >>> sc = SkyCoord(1*u.deg, 2*u.deg, 3*u.km,
     ...               frame="heliographic_carrington",
+    ...               observer="earth",
     ...               obstime="2010/01/01T00:00:30")
     >>> sc
-    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:30.000): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:30.000, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (lon, lat, radius) in (deg, deg, km)
         (1., 2., 3.)>
 
     >>> sc = SkyCoord([1,2,3]*u.deg, [4,5,6]*u.deg, [5,6,7]*u.km,
     ...               obstime="2010/01/01T00:00:45", frame="heliographic_carrington")
     >>> sc
-    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:45.000): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicCarrington: obstime=2010-01-01T00:00:45.000, observer=None): (lon, lat, radius) in (deg, deg, km)
         [(1., 4., 5.), (2., 5., 6.), (3., 6., 7.)]>
 
     >>> sc = SkyCoord(CartesianRepresentation(0*u.km, 45*u.km, 2*u.km),
     ...               obstime="2011/01/05T00:00:50",
     ...               frame="heliographic_carrington")
     >>> sc
-    <SkyCoord (HeliographicCarrington: obstime=2011-01-05T00:00:50.000): (lon, lat, radius) in (deg, deg, km)
+    <SkyCoord (HeliographicCarrington: obstime=2011-01-05T00:00:50.000, observer=None): (lon, lat, radius) in (deg, deg, km)
         (90., 2.54480438, 45.04442252)>
     """
     name = "heliographic_carrington"
     _wrap_angle = 360*u.deg
 
+    observer = ObserverCoordinateAttribute(HeliographicStonyhurst)
+
 
 @add_common_docstring(**_frame_parameters())
 class Heliocentric(SunPyBaseCoordinateFrame):
@@ -292,11 +319,11 @@ class Heliocentric(SunPyBaseCoordinateFrame):
     by 90 degrees compared to the convention of the Heliocentric Radial (HCR) system.
 
     A new instance can be created using the following signatures
-    (note that ``obstime`` and ``representation_type`` must be supplied as
-    keywords)::
+    (note that if supplied, ``obstime``, ``observer``, and ``representation_type`` must be
+    keyword arguments)::
 
-        Heliocentric(x, y, z, obstime)
-        Heliocentric(rho, psi, z, obstime, representation_type='cylindrical')
+        Heliocentric(x, y, z, obstime=obstime, observer=observer)
+        Heliocentric(rho, psi, z, representation_type='cylindrical', obstime=obstime, observer=observer)
 
     Parameters
     ----------
@@ -313,20 +340,19 @@ class Heliocentric(SunPyBaseCoordinateFrame):
     >>> import astropy.units as u
 
     >>> sc = SkyCoord(CartesianRepresentation(10*u.km, 1*u.km, 2*u.km),
-    ...               obstime="2011/01/05T00:00:50", frame="heliocentric")
+    ...               obstime="2011/01/05T00:00:50", observer="earth", frame="heliocentric")
     >>> sc
     <SkyCoord (Heliocentric: obstime=2011-01-05T00:00:50.000, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (x, y, z) in km
         (10., 1., 2.)>
 
-    >>> sc = SkyCoord([1,2]*u.km, [3,4]*u.m, [5,6]*u.cm, frame="heliocentric", obstime="2011/01/01T00:00:54")
-
+    >>> sc = SkyCoord([1,2]*u.km, [3,4]*u.m, [5,6]*u.cm,
+    ...               obstime="2011/01/01T00:00:54", observer="earth", frame="heliocentric")
     >>> sc
     <SkyCoord (Heliocentric: obstime=2011-01-01T00:00:54.000, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (x, y, z) in (km, m, cm)
         [(1., 3., 5.), (2., 4., 6.)]>
 
     >>> sc = SkyCoord(CylindricalRepresentation(10*u.km, 60*u.deg, 10*u.km),
-    ...               obstime="2011/01/05T00:00:50",
-    ...               frame="heliocentric")
+    ...               obstime="2011/01/05T00:00:50", observer="earth", frame="heliocentric")
     >>> sc
     <SkyCoord (Heliocentric: obstime=2011-01-05T00:00:50.000, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (x, y, z) in km
         (5., 8.66025404, 10.)>
@@ -337,7 +363,7 @@ class Heliocentric(SunPyBaseCoordinateFrame):
         CylindricalRepresentation: [RepresentationMapping('phi', 'psi', u.deg)]
     }
 
-    observer = ObserverCoordinateAttribute(HeliographicStonyhurst, default="earth")
+    observer = ObserverCoordinateAttribute(HeliographicStonyhurst)
 
 
 @add_common_docstring(**_frame_parameters())
@@ -357,10 +383,10 @@ class Helioprojective(SunPyBaseCoordinateFrame):
     is appropriate, ``theta_x`` and ``theta_y`` can be approximated as Cartesian components.
 
     A new instance can be created using the following signatures
-    (note that ``obstime`` and ``observer`` must be supplied as keywords)::
+    (note that if supplied, ``obstime`` and ``observer`` must be keyword arguments)::
 
-        Helioprojective(theta_x, theta_y, obstime, observer)
-        Helioprojective(theta_x, theta_y, distance, obstime, observer)
+        Helioprojective(theta_x, theta_y, obstime=obstime, observer=observer)
+        Helioprojective(theta_x, theta_y, distance, obstime=obstime, observer=observer)
 
     Parameters
     ----------
@@ -384,18 +410,18 @@ class Helioprojective(SunPyBaseCoordinateFrame):
     >>> from astropy.coordinates import SkyCoord
     >>> import sunpy.coordinates
     >>> import astropy.units as u
-    >>> sc = SkyCoord(0*u.deg, 0*u.deg, 5*u.km, obstime="2010/01/01T00:00:00",
-    ...               frame="helioprojective")
+    >>> sc = SkyCoord(0*u.deg, 0*u.deg, 5*u.km,
+    ...               obstime="2010/01/01T00:00:00", observer="earth", frame="helioprojective")
     >>> sc
     <SkyCoord (Helioprojective: obstime=2010-01-01T00:00:00.000, rsun=695700.0 km, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (Tx, Ty, distance) in (arcsec, arcsec, km)
         (0., 0., 5.)>
-    >>> sc = SkyCoord(0*u.deg, 0*u.deg, obstime="2010/01/01T00:00:00", frame="helioprojective")
+    >>> sc = SkyCoord(0*u.deg, 0*u.deg,
+    ...               obstime="2010/01/01T00:00:00", observer="earth", frame="helioprojective")
     >>> sc
     <SkyCoord (Helioprojective: obstime=2010-01-01T00:00:00.000, rsun=695700.0 km, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (Tx, Ty) in arcsec
         (0., 0.)>
     >>> sc = SkyCoord(CartesianRepresentation(1*u.AU, 1e5*u.km, -2e5*u.km),
-    ...               obstime="2011/01/05T00:00:50",
-    ...               frame="helioprojective")
+    ...               obstime="2011/01/05T00:00:50", observer="earth", frame="helioprojective")
     >>> sc
     <SkyCoord (Helioprojective: obstime=2011-01-05T00:00:50.000, rsun=695700.0 km, observer=<HeliographicStonyhurst Coordinate for 'earth'>): (Tx, Ty, distance) in (arcsec, arcsec, AU)
         (137.87948623, -275.75878762, 1.00000112)>
@@ -422,19 +448,18 @@ class Helioprojective(SunPyBaseCoordinateFrame):
     }
 
     rsun = Attribute(default=_RSUN.to(u.km))
-    observer = ObserverCoordinateAttribute(HeliographicStonyhurst, default="earth")
+    observer = ObserverCoordinateAttribute(HeliographicStonyhurst)
 
-    def calculate_distance(self):
+    def make_3d(self):
         """
         This method calculates the third coordinate of the Helioprojective
-        frame. It assumes that the coordinate point is on the disk of the Sun
-        at the rsun radius.
+        frame. It assumes that the coordinate point is on the surface of the Sun.
 
         If a point in the frame is off limb then NaN will be returned.
 
         Returns
         -------
-        new_frame : `~sunpy.coordinates.frames.HelioProjective`
+        new_frame : `~sunpy.coordinates.frames.Helioprojective`
             A new frame instance with all the attributes of the original but
             now with a third coordinate.
         """
@@ -444,9 +469,9 @@ class Helioprojective(SunPyBaseCoordinateFrame):
             return self
 
         if not isinstance(self.observer, BaseCoordinateFrame):
-            raise ConvertError("Cannot calculate distance to the solar disk "
-                               "for observer '{}' "
-                               "without `obstime` being specified.".format(self.observer))
+            raise ConvertError("Cannot calculate distance to the Sun "
+                               f"for observer '{self.observer}' "
+                               "without `obstime` being specified.")
 
         rep = self.represent_as(UnitSphericalRepresentation)
         lat, lon = rep.lat, rep.lon
@@ -460,3 +485,104 @@ class Helioprojective(SunPyBaseCoordinateFrame):
         return self.realize_frame(SphericalRepresentation(lon=lon,
                                                           lat=lat,
                                                           distance=d))
+
+    # Support the previous name for make_3d for now
+    calculate_distance = deprecated('1.1', name="calculate_distance",
+                                    alternative="make_3d")(make_3d)
+
+
+@add_common_docstring(**_frame_parameters())
+class HeliocentricEarthEcliptic(SunPyBaseCoordinateFrame):
+    """
+    A coordinate or frame in the Heliocentric Earth Ecliptic (HEE) system.
+
+    - The origin is the center of the Sun.
+    - The X-axis (0 degrees longitude and 0 degrees latitude) is aligned with the Sun-Earth line.
+    - The Z-axis (+90 degrees latitude) is aligned with the component perpendicular to the X-axis
+      of the mean ecliptic pole at the observation time.
+
+    Parameters
+    ----------
+    {data}
+    {lonlat}
+    {distance_sun}
+    {common}
+    """
+    default_representation = SphericalRepresentation
+
+
+@add_common_docstring(**_frame_parameters())
+class GeocentricSolarEcliptic(SunPyBaseCoordinateFrame):
+    """
+    A coordinate or frame in the Geocentric Solar Ecliptic (GSE) system.
+
+    - The origin is the center of the Earth.
+    - The X-axis (0 degrees longitude and 0 degrees latitude) is aligned with the Earth-Sun line.
+    - The Z-axis (+90 degrees latitude) is aligned with the component perpendicular to the X-axis
+      of the mean ecliptic pole at the observation time.
+
+    Parameters
+    ----------
+    {data}
+    {lonlat}
+    {distance_earth}
+    {common}
+
+    Notes
+    -----
+    Aberration due to Earth motion is not included.
+    """
+    default_representation = SphericalRepresentation
+
+
+@add_common_docstring(**_frame_parameters())
+class HeliocentricInertial(SunPyBaseCoordinateFrame):
+    """
+    A coordinate or frame in the Heliocentric Inertial (HCI) system.
+
+    - The origin is the center of the Sun.
+    - The Z-axis (+90 degrees latitude) is aligned with the Sun's north pole.
+    - The X-axis (0 degrees longitude and 0 degrees latitude) is aligned with the solar ascending
+      node on the ecliptic (mean J2000.0).
+
+    Parameters
+    ----------
+    {data}
+    {lonlat}
+    {distance_sun}
+    {common}
+
+    Notes
+    -----
+    The solar ascending node on the ecliptic lies on the intersection of the solar equatorial
+    plane with the ecliptic plane, not on the intersection of the celestial equatorial plane with
+    the ecliptic plane.
+    """
+    default_representation = SphericalRepresentation
+
+
+@add_common_docstring(**_frame_parameters())
+class GeocentricEarthEquatorial(SunPyBaseCoordinateFrame):
+    """
+    A coordinate or frame in the Geocentric Earth Equatorial (GEI) system.
+
+    - The origin is the center of the Earth.
+    - The Z-axis (+90 degrees latitude) is aligned with the Earth's north pole.
+    - The X-axis (0 degrees longitude and 0 degrees latitude) is aligned with the mean (not true)
+      vernal equinox.
+
+    Parameters
+    ----------
+    {data}
+    {lonlat}
+    {distance_earth}
+    {equinox}
+    {common}
+
+    Notes
+    -----
+    Aberration due to Earth motion is not included.
+    """
+    default_representation = SphericalRepresentation
+
+    equinox = TimeFrameAttributeSunPy(default=_J2000)
