@@ -1,16 +1,24 @@
-# -*- coding: utf-8 -*-
 
 import pytest
+from hypothesis import given, settings
 
 import astropy.units as u
-from astropy.config.paths import set_temp_cache
 from astropy.constants import c as speed_of_light
-from astropy.coordinates import SkyCoord
+from astropy.coordinates import SkyCoord, solar_system_ephemeris
 from astropy.tests.helper import assert_quantity_allclose
 from astropy.time import Time
 
 from sunpy.coordinates.ephemeris import *
-from sunpy.time import parse_time
+from .strategies import times
+
+
+@pytest.fixture(scope='function')
+def astropy_ephemeris_de432s():
+    # Temporarily set Astropy's ephemeris to DE432s
+    old_ephemeris = solar_system_ephemeris.get()
+    solar_system_ephemeris.set('de432s')
+    yield solar_system_ephemeris.get()
+    solar_system_ephemeris.set(old_ephemeris)
 
 
 def test_get_body_heliographic_stonyhurst():
@@ -27,7 +35,7 @@ def test_get_body_heliographic_stonyhurst():
 
 
 def test_get_body_heliographic_stonyhurst_light_travel_time():
-    # Tests whether the apparent position of the Sun accoutns for light travel time
+    # Tests whether the apparent position of the Sun accounts for light travel time
     t = Time('2012-06-05 22:34:48.350')  # An arbitrary test time
 
     # Use the implemented correction for light travel time
@@ -43,6 +51,23 @@ def test_get_body_heliographic_stonyhurst_light_travel_time():
     assert_quantity_allclose(difference, 0*u.m, atol=1*u.m)
 
 
+def test_get_body_heliographic_stonyhurst_light_travel_time_array():
+    # Tests whether requesting an array of locations returns the same answers as individually
+    t1 = Time('2001-02-03 04:05:06')
+    t2 = Time('2011-12-13 14:15:16')
+
+    venus1 = get_body_heliographic_stonyhurst('venus', t1, observer=get_earth(t1))
+    venus2 = get_body_heliographic_stonyhurst('venus', t2, observer=get_earth(t2))
+    both = get_body_heliographic_stonyhurst('venus', [t1, t2], observer=get_earth([t1, t2]))
+
+    assert_quantity_allclose(venus1.lon, both[0].lon)
+    assert_quantity_allclose(venus1.lat, both[0].lat)
+    assert_quantity_allclose(venus1.radius, both[0].radius)
+    assert_quantity_allclose(venus2.lon, both[1].lon)
+    assert_quantity_allclose(venus2.lat, both[1].lat)
+    assert_quantity_allclose(venus2.radius, both[1].radius)
+
+
 def test_get_earth():
     # Validate against published values from the Astronomical Almanac (2013)
     e1 = get_earth('2013-Jan-01')
@@ -59,7 +84,7 @@ def test_get_earth():
 @pytest.mark.remote_data
 def test_get_horizons_coord():
     # get_horizons_coord() depends on astroquery
-    astroquery = pytest.importorskip("astroquery")
+    pytest.importorskip("astroquery")
 
     # Validate against published values from the Astronomical Almanac (2013)
     e1 = get_horizons_coord('Geocenter', '2013-Jan-01')
@@ -76,7 +101,7 @@ def test_get_horizons_coord():
 @pytest.mark.remote_data
 def test_get_horizons_coord_array_time():
     # get_horizons_coord() depends on astroquery
-    astroquery = pytest.importorskip("astroquery")
+    pytest.importorskip("astroquery")
 
     # Validate against published values from the Astronomical Almanac (2013, C8-C13)
     array_time = Time(['2013-05-01', '2013-06-01', '2013-04-01', '2013-03-01'])
@@ -100,12 +125,18 @@ def test_get_horizons_coord_array_time():
 
 
 @pytest.mark.remote_data
-def test_consistency_with_horizons():
+@given(obstime=times())
+@settings(deadline=5000, max_examples=10)
+def test_consistency_with_horizons(astropy_ephemeris_de432s, obstime):
     # get_horizons_coord() depends on astroquery
-    astroquery = pytest.importorskip("astroquery")
+    pytest.importorskip("astroquery")
 
     # Check whether the location of Earth is the same between Astropy and JPL HORIZONS
-    now = parse_time('now')
-    e1 = get_earth(now)
-    e2 = get_horizons_coord('Geocenter', now)
-    assert_quantity_allclose(e1.separation_3d(e2), 0*u.km, atol=35*u.km)
+    e1 = get_earth(obstime)
+    e2 = get_horizons_coord('Geocenter', obstime)
+    assert_quantity_allclose(e2.separation_3d(e1), 0*u.km, atol=50*u.m)
+
+    # Check whether the location of Mars is the same between Astropy and JPL HORIZONS
+    e1 = get_body_heliographic_stonyhurst('mars', obstime)
+    e2 = get_horizons_coord('Mars barycenter', obstime)
+    assert_quantity_allclose(e2.separation_3d(e1), 0*u.km, atol=500*u.m)
