@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 Coordinate Transformation Functions
 
@@ -16,46 +15,56 @@ This module contains the functions for converting one
 import logging
 from copy import deepcopy
 from functools import wraps
+from contextlib import contextmanager
 
 import numpy as np
 
 import astropy.units as u
 from astropy._erfa import obl06
-from astropy.coordinates import (HCRS, ICRS, BaseCoordinateFrame, ConvertError,
-                                 get_body_barycentric, get_body_barycentric_posvel)
+from astropy.constants import c as speed_of_light
+from astropy.coordinates import (
+    HCRS,
+    ICRS,
+    BaseCoordinateFrame,
+    ConvertError,
+    HeliocentricMeanEcliptic,
+    get_body_barycentric,
+    get_body_barycentric_posvel,
+)
 from astropy.coordinates.baseframe import frame_transform_graph
+from astropy.coordinates.builtin_frames import make_transform_graph_docs
 from astropy.coordinates.builtin_frames.utils import get_jd12
 from astropy.coordinates.matrix_utilities import matrix_product, matrix_transpose, rotation_matrix
-from astropy.coordinates.representation import (CartesianDifferential, CartesianRepresentation,
-                                                SphericalRepresentation,
-                                                UnitSphericalRepresentation)
-from astropy.coordinates.transformations import (AffineTransform, FunctionTransform,
-                                                 FunctionTransformWithFiniteDifference)
+from astropy.coordinates.representation import (
+    CartesianDifferential,
+    CartesianRepresentation,
+    SphericalRepresentation,
+    UnitSphericalRepresentation,
+)
+from astropy.coordinates.transformations import (
+    AffineTransform,
+    FunctionTransform,
+    FunctionTransformWithFiniteDifference,
+)
 
 from sunpy import log
 from sunpy.sun import constants
-
-from .frames import (_J2000, GeocentricEarthEquatorial, GeocentricSolarEcliptic,
-                     Heliocentric, HeliocentricEarthEcliptic, HeliocentricInertial,
-                     HeliographicCarrington, HeliographicStonyhurst, Helioprojective)
-
-# Versions of Astropy that do not have HeliocentricMeanEcliptic have the same frame
-# with the incorrect name HeliocentricTrueEcliptic
-try:
-    from astropy.coordinates import HeliocentricMeanEcliptic
-except ImportError:
-    from astropy.coordinates import HeliocentricTrueEcliptic as HeliocentricMeanEcliptic
-
-try:
-    from astropy.coordinates.builtin_frames import _make_transform_graph_docs as make_transform_graph_docs
-except ImportError:
-    from astropy.coordinates import make_transform_graph_docs as _make_transform_graph_docs
-    make_transform_graph_docs = lambda: _make_transform_graph_docs(frame_transform_graph)
-
+from .frames import (
+    _J2000,
+    GeocentricEarthEquatorial,
+    GeocentricSolarEcliptic,
+    Heliocentric,
+    HeliocentricEarthEcliptic,
+    HeliocentricInertial,
+    HeliographicCarrington,
+    HeliographicStonyhurst,
+    Helioprojective,
+)
 
 RSUN_METERS = constants.get('radius').si.to(u.m)
 
-__all__ = ['hgs_to_hgc', 'hgc_to_hgs', 'hcc_to_hpc',
+__all__ = ['transform_with_sun_center',
+           'hgs_to_hgc', 'hgc_to_hgs', 'hcc_to_hpc',
            'hpc_to_hcc', 'hcc_to_hgs', 'hgs_to_hcc',
            'hpc_to_hpc',
            'hcrs_to_hgs', 'hgs_to_hcrs',
@@ -66,6 +75,71 @@ __all__ = ['hgs_to_hgc', 'hgc_to_hgs', 'hcc_to_hpc',
            'hme_to_gei', 'gei_to_hme', 'gei_to_gei']
 
 
+# Boolean flag for whether to ignore the motion of the center of the Sun in inertial space
+_ignore_sun_motion = False
+
+
+@contextmanager
+def transform_with_sun_center():
+    """
+    Context manager for coordinate transformations to ignore the motion of the center of the Sun.
+
+    Normally, coordinates refer to a point in inertial space (relative to the barycenter of the
+    solar system).  Transforming to a different observation time does not move the point at all,
+    but rather only updates the coordinate representation as needed for the origin and axis
+    orientations at the new observation time.  However, the center of the Sun moves over time.
+    Thus, for example, a coordinate that lies on the surface of the Sun at one observation time
+    will not continue to lie on the surface of the Sun at other observation times.
+
+    Under this context manager, transformations will instead move the coordinate over time to
+    "follow" the translational motion of the center of Sun, thus maintaining the position of the
+    coordinate relative to the center of the Sun.
+
+    Examples
+    --------
+    >>> from astropy.coordinates import SkyCoord
+    >>> from sunpy.coordinates import HeliographicStonyhurst, transform_with_sun_center
+    >>> import astropy.units as u
+    >>> start_frame = HeliographicStonyhurst(obstime="2001-01-01")
+    >>> end_frame = HeliographicStonyhurst(obstime="2001-02-01")
+    >>> sun_center = SkyCoord(0*u.deg, 0*u.deg, 0*u.AU, frame=start_frame)
+    >>> sun_center
+    <SkyCoord (HeliographicStonyhurst: obstime=2001-01-01T00:00:00.000): (lon, lat, radius) in (deg, deg, AU)
+        (0., 0., 0.)>
+    >>> sun_center.transform_to(end_frame)  # transformations do not normally follow Sun center
+    <SkyCoord (HeliographicStonyhurst: obstime=2001-02-01T00:00:00.000): (lon, lat, radius) in (deg, deg, AU)
+        (-156.66825767, 5.96399877, 0.00027959)>
+    >>> with transform_with_sun_center():
+    ...     sun_center.transform_to(end_frame)  # now following Sun center
+    <SkyCoord (HeliographicStonyhurst: obstime=2001-02-01T00:00:00.000): (lon, lat, radius) in (deg, deg, AU)
+        (0., 0., 0.)>
+
+    Notes
+    -----
+    This context manager accounts only for the motion of the center of the Sun, i.e.,
+    translational motion.  The motion of solar features due to any rotation of the Sun about its
+    rotational axis is not accounted for.
+
+    Due to the implementation approach, this context manager modifies transformations between only
+    these five coordinate frames:
+    `~sunpy.coordinates.frames.HeliographicStonyhurst`,
+    `~sunpy.coordinates.frames.HeliographicCarrington`,
+    `~sunpy.coordinates.frames.HeliocentricInertial`,
+    `~sunpy.coordinates.frames.Heliocentric`, and
+    `~sunpy.coordinates.frames.Helioprojective`.
+    """
+    try:
+        global _ignore_sun_motion
+
+        old_ignore_sun_motion = _ignore_sun_motion  # nominally False
+
+        log.debug("Ignore the motion of the center of the Sun for transformations")
+        _ignore_sun_motion = True
+        yield
+    finally:
+        _ignore_sun_motion = old_ignore_sun_motion
+
+
 # Global counter to keep track of the layer of transformation
 _layer_level = 0
 
@@ -118,7 +192,7 @@ def _transformation_debug(description):
 
 def _observers_are_equal(obs_1, obs_2):
     # Note that this also lets pass the situation where both observers are None
-    if obs_1 == obs_2:
+    if obs_1 is obs_2:
         return True
 
     # obs_1 != obs_2
@@ -174,19 +248,26 @@ def _transform_obstime(frame, obstime):
         return new_frame
 
 
-def _rotation_matrix_hgs_to_hgc(obstime):
+def _rotation_matrix_hgs_to_hgc(obstime, observer_distance_from_sun):
     """
     Return the rotation matrix from HGS to HGC at the same observation time
     """
     if obstime is None:
-        raise ValueError("To perform this transformation the coordinate"
-                         " Frame needs an obstime Attribute")
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
 
     # Import here to avoid a circular import
-    from .sun import L0
+    from .sun import L0, earth_distance
+
+    # Calculate the difference in light travel time if the observer is at a different distance from
+    # the Sun than the Earth is
+    delta_time = (observer_distance_from_sun - earth_distance(obstime)) / speed_of_light
+
+    # Calculate the corresponding difference in apparent longitude
+    delta_lon = delta_time * constants.sidereal_rotation_rate
 
     # Rotation is only in longitude, so only around the Z axis
-    return rotation_matrix(-L0(obstime), 'z')
+    return rotation_matrix(-(L0(obstime) + delta_lon), 'z')
 
 
 @frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
@@ -196,11 +277,13 @@ def hgs_to_hgc(hgscoord, hgcframe):
     """
     Convert from Heliographic Stonyhurst to Heliographic Carrington.
     """
+    _check_observer_defined(hgcframe)
+
     # First transform the HGS coord to the HGC obstime
     int_coord = _transform_obstime(hgscoord, hgcframe.obstime)
 
     # Rotate from HGS to HGC
-    total_matrix = _rotation_matrix_hgs_to_hgc(int_coord.obstime)
+    total_matrix = _rotation_matrix_hgs_to_hgc(int_coord.obstime, hgcframe.observer.radius)
     newrepr = int_coord.cartesian.transform(total_matrix)
 
     return hgcframe._replicate(newrepr, obstime=int_coord.obstime)
@@ -213,11 +296,14 @@ def hgc_to_hgs(hgccoord, hgsframe):
     """
     Convert from Heliographic Carrington to Heliographic Stonyhurst.
     """
+    _check_observer_defined(hgccoord)
+
     # First transform the HGC coord to the HGS obstime
     int_coord = _transform_obstime(hgccoord, hgsframe.obstime)
 
     # Rotate from HGC to HGS
-    total_matrix = matrix_transpose(_rotation_matrix_hgs_to_hgc(int_coord.obstime))
+    total_matrix = matrix_transpose(_rotation_matrix_hgs_to_hgc(int_coord.obstime,
+                                                                hgccoord.observer.radius))
     newrepr = int_coord.cartesian.transform(total_matrix)
 
     return hgsframe._replicate(newrepr, obstime=int_coord.obstime)
@@ -430,7 +516,8 @@ def _sun_earth_icrf(time):
 # (See Archinal et al. 2011,
 #   "Report of the IAU Working Group on Cartographic Coordinates and Rotational Elements: 2009")
 # The orientation of the north pole in ICRS/HCRS is assumed to be constant in time
-_SOLAR_NORTH_POLE_HCRS = UnitSphericalRepresentation(lon=286.13*u.deg, lat=63.87*u.deg)
+_SOLAR_NORTH_POLE_HCRS = UnitSphericalRepresentation(lon=constants.get('alpha_0'),
+                                                     lat=constants.get('delta_0'))
 
 
 # Calculate the rotation matrix to de-tilt the Sun's rotation axis to be parallel to the Z axis
@@ -453,8 +540,8 @@ def hcrs_to_hgs(hcrscoord, hgsframe):
     vector.
     """
     if hgsframe.obstime is None:
-        raise ValueError("To perform this transformation the coordinate"
-                         " Frame needs an obstime Attribute")
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
 
     # Check whether differentials are involved on either end
     has_differentials = ((hcrscoord._data is not None and hcrscoord.data.differentials) or
@@ -481,7 +568,7 @@ def hcrs_to_hgs(hcrscoord, hgsframe):
 
     # All of the above is calculated for the HGS observation time
     # If the HCRS observation time is different, calculate the translation in origin
-    if np.any(hcrscoord.obstime != hgsframe.obstime):
+    if not _ignore_sun_motion and np.any(hcrscoord.obstime != hgsframe.obstime):
         sun_pos_old_icrs = get_body_barycentric('sun', hcrscoord.obstime)
         offset_icrf = sun_pos_icrs - sun_pos_old_icrs
     else:
@@ -541,11 +628,17 @@ def hgc_to_hgc(from_coo, to_frame):
     """
     Convert between two Heliographic Carrington frames.
     """
-    if np.all(from_coo.obstime == to_frame.obstime):
+    if _observers_are_equal(from_coo.observer, to_frame.observer) and \
+       np.all(from_coo.obstime == to_frame.obstime):
         return to_frame.realize_frame(from_coo.data)
-    else:
-        return from_coo.transform_to(HeliographicStonyhurst(obstime=from_coo.obstime)).\
-               transform_to(to_frame)
+
+    _check_observer_defined(from_coo)
+    _check_observer_defined(to_frame)
+
+    # Convert through HGS
+    hgscoord = from_coo.transform_to(HeliographicStonyhurst(obstime=from_coo.obstime))
+
+    return hgscoord.transform_to(to_frame)
 
 
 @frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
@@ -593,6 +686,10 @@ def hme_to_hee(hmecoord, heeframe):
     """
     Convert from Heliocentric Mean Ecliptic to Heliocentric Earth Ecliptic
     """
+    if heeframe.obstime is None:
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
+
     # Convert to the HME frame with mean equinox of date at the HEE obstime, through HCRS
     int_frame = HeliocentricMeanEcliptic(obstime=heeframe.obstime, equinox=heeframe.obstime)
     int_coord = hmecoord.transform_to(HCRS).transform_to(int_frame)
@@ -611,6 +708,10 @@ def hee_to_hme(heecoord, hmeframe):
     """
     Convert from Heliocentric Earth Ecliptic to Heliocentric Mean Ecliptic
     """
+    if heecoord.obstime is None:
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
+
     int_frame = HeliocentricMeanEcliptic(obstime=heecoord.obstime, equinox=heecoord.obstime)
 
     # Rotate the HEE coord to the intermediate frame
@@ -631,6 +732,8 @@ def hee_to_hee(from_coo, to_frame):
     """
     if np.all(from_coo.obstime == to_frame.obstime):
         return to_frame.realize_frame(from_coo.data)
+    elif to_frame.obstime is None:
+        return from_coo
     else:
         return from_coo.transform_to(HCRS).transform_to(to_frame)
 
@@ -642,22 +745,25 @@ def hee_to_gse(heecoord, gseframe):
     """
     Convert from Heliocentric Earth Ecliptic to Geocentric Solar Ecliptic
     """
-    # Use an intermediate frame of HEE at the GSE observation time
-    int_frame = HeliocentricEarthEcliptic(obstime=gseframe.obstime)
-    int_coord = heecoord.transform_to(int_frame)
+    # First transform the HEE coord to the GSE obstime
+    int_coord = _transform_obstime(heecoord, gseframe.obstime)
 
-    # Get the Sun-Earth vector in the intermediate frame
-    sun_earth = HCRS(_sun_earth_icrf(int_frame.obstime), obstime=int_frame.obstime)
-    sun_earth_int = sun_earth.transform_to(int_frame).cartesian
+    if int_coord.obstime is None:
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
+
+    # Import here to avoid a circular import
+    from .sun import earth_distance
 
     # Find the Earth-object vector in the intermediate frame
+    sun_earth_int = earth_distance(int_coord.obstime) * CartesianRepresentation(1, 0, 0)
     earth_object_int = int_coord.cartesian - sun_earth_int
 
     # Flip the vector in X and Y, but leave Z untouched
     # (The additional transpose operations are to handle both scalar and array inputs)
     newrepr = CartesianRepresentation((earth_object_int.xyz.T * [-1, -1, 1]).T)
 
-    return gseframe.realize_frame(newrepr)
+    return gseframe._replicate(newrepr, obstime=int_coord.obstime)
 
 
 @frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
@@ -667,23 +773,25 @@ def gse_to_hee(gsecoord, heeframe):
     """
     Convert from Geocentric Solar Ecliptic to Heliocentric Earth Ecliptic
     """
-    # Use an intermediate frame of HEE at the GSE observation time
-    int_frame = HeliocentricEarthEcliptic(obstime=gsecoord.obstime)
+    # First transform the GSE coord to the HEE obstime
+    int_coord = _transform_obstime(gsecoord, heeframe.obstime)
 
-    # Get the Sun-Earth vector in the intermediate frame
-    sun_earth = HCRS(_sun_earth_icrf(int_frame.obstime), obstime=int_frame.obstime)
-    sun_earth_int = sun_earth.transform_to(int_frame).cartesian
+    if int_coord.obstime is None:
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
 
-    # Find the Earth-object vector in the intermediate frame
-    # Flip the vector in X and Y, but leave Z untouched
-    # (The additional transpose operations are to handle both scalar and array inputs)
-    earth_object_int = CartesianRepresentation((gsecoord.cartesian.xyz.T * [-1, -1, 1]).T)
+    # Import here to avoid a circular import
+    from .sun import earth_distance
 
     # Find the Sun-object vector in the intermediate frame
-    sun_object_int = sun_earth_int + earth_object_int
-    int_coord = int_frame.realize_frame(sun_object_int)
+    earth_sun_int = earth_distance(int_coord.obstime) * CartesianRepresentation(1, 0, 0)
+    sun_object_int = int_coord.cartesian - earth_sun_int
 
-    return int_coord.transform_to(heeframe)
+    # Flip the vector in X and Y, but leave Z untouched
+    # (The additional transpose operations are to handle both scalar and array inputs)
+    newrepr = CartesianRepresentation((sun_object_int.xyz.T * [-1, -1, 1]).T)
+
+    return heeframe._replicate(newrepr, obstime=int_coord.obstime)
 
 
 @frame_transform_graph.transform(FunctionTransformWithFiniteDifference,
@@ -729,6 +837,10 @@ def hgs_to_hci(hgscoord, hciframe):
     # First transform the HGS coord to the HCI obstime
     int_coord = _transform_obstime(hgscoord, hciframe.obstime)
 
+    if int_coord.obstime is None:
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
+
     # Rotate from HGS to HCI
     total_matrix = _rotation_matrix_hgs_to_hci(int_coord.obstime)
     newrepr = int_coord.cartesian.transform(total_matrix)
@@ -746,6 +858,10 @@ def hci_to_hgs(hcicoord, hgsframe):
     # First transform the HCI coord to the HGS obstime
     int_coord = _transform_obstime(hcicoord, hgsframe.obstime)
 
+    if int_coord.obstime is None:
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
+
     # Rotate from HCI to HGS
     total_matrix = matrix_transpose(_rotation_matrix_hgs_to_hci(int_coord.obstime))
     newrepr = int_coord.cartesian.transform(total_matrix)
@@ -764,7 +880,7 @@ def hci_to_hci(from_coo, to_frame):
         return to_frame.realize_frame(from_coo.data)
     else:
         return from_coo.transform_to(HeliographicStonyhurst(obstime=from_coo.obstime)).\
-               transform_to(to_frame)
+            transform_to(to_frame)
 
 
 def _rotation_matrix_obliquity(time):
@@ -781,6 +897,10 @@ def hme_to_gei(hmecoord, geiframe):
     """
     Convert from Heliocentric Mean Ecliptic to Geocentric Earth Equatorial
     """
+    if geiframe.obstime is None:
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
+
     # Use an intermediate frame of HME at the GEI observation time, through HCRS
     int_frame = HeliocentricMeanEcliptic(obstime=geiframe.obstime, equinox=geiframe.equinox)
     int_coord = hmecoord.transform_to(HCRS).transform_to(int_frame)
@@ -806,6 +926,10 @@ def gei_to_hme(geicoord, hmeframe):
     """
     Convert from Geocentric Earth Equatorial to Heliocentric Mean Ecliptic
     """
+    if geicoord.obstime is None:
+        raise ConvertError("To perform this transformation, the coordinate"
+                           " frame needs a specified `obstime`.")
+
     # Use an intermediate frame of HME at the GEI observation time
     int_frame = HeliocentricMeanEcliptic(obstime=geicoord.obstime, equinox=geicoord.equinox)
 
@@ -851,9 +975,6 @@ def _make_sunpy_graph():
                  'gcrs', 'precessedgeocentric', 'geocentrictrueecliptic', 'geocentricmeanecliptic',
                  'cirs', 'altaz', 'itrs']
 
-    global frame_transform_graph
-    backup_graph = deepcopy(frame_transform_graph)
-
     small_graph = deepcopy(frame_transform_graph)
     cull_list = [name for name in small_graph.get_names() if name not in keep_list]
     cull_frames = [small_graph.lookup_name(name) for name in cull_list]
@@ -874,13 +995,7 @@ def _make_sunpy_graph():
 
     _add_astropy_node(small_graph)
 
-    # Overwrite the main transform graph
-    frame_transform_graph = small_graph
-
-    docstr = make_transform_graph_docs()
-
-    # Restore the main transform graph
-    frame_transform_graph = backup_graph
+    docstr = make_transform_graph_docs(small_graph)
 
     # Make adjustments to the graph
     docstr = _tweak_graph(docstr)
