@@ -1,16 +1,15 @@
-#
-# Calculates the co-ordinates along great arcs between two specified points
-# which are assumed to be on disk.
-#
+"""
+Miscellaneous utilities related to coordinates
+"""
+
 import numpy as np
 
 import astropy.units as u
-from astropy.coordinates import SkyCoord
+from astropy.coordinates import BaseCoordinateFrame, SkyCoord
 
 from sunpy.coordinates import Heliocentric
 
-
-__all__ = ['GreatArc']
+__all__ = ['GreatArc', 'get_rectangle_coordinates']
 
 
 class GreatArc:
@@ -265,3 +264,123 @@ class GreatArc:
                         obstime=self.obstime,
                         observer=self.observer,
                         frame=Heliocentric).transform_to(self.start_frame)
+
+
+def get_rectangle_coordinates(bottom_left, *, top_right=None,
+                              width: u.deg = None, height: u.deg = None):
+    """
+    Specify a rectangular region of interest in longitude and latitude in a given coordinate frame.
+
+    Parameters
+    ----------
+    bottom_left : `~astropy.coordinates.BaseCoordinateFrame` or `~astropy.coordinates.SkyCoord`
+        The bottom-left coordinate of the rectangle. Supports passing both the
+        bottom left and top right coordinates by passing with a shape of ``(2,)``.
+    top_right : `~astropy.coordinates.BaseCoordinateFrame` or `~astropy.coordinates.SkyCoord`
+        The top-right coordinate of the rectangle.
+        If in a different frame than ``bottom_left`` and all required metadata
+        for frame conversion is present, ``top_right`` will be transformed to
+        ``bottom_left`` frame.
+    width : `~astropy.units.Quantity`
+        The width of the rectangle.
+        Must be omitted if the coordinates of both corners have been specified.
+    height : `~astropy.units.Quantity`
+        The height of the rectangle.
+        Must be omitted if the coordinates of both corners have been specified.
+
+    Returns
+    -------
+    `~astropy.coordinates.BaseCoordinateFrame` or `~astropy.coordinates.SkyCoord`
+        The bottom left coordinate of the rectangular region of interest.
+    `~astropy.coordinates.BaseCoordinateFrame` or `~astropy.coordinates.SkyCoord`
+        The top right coordinate of the rectangular region of interest.
+
+    Examples
+    --------
+    >>> import astropy.units as u
+    >>> from astropy.coordinates import SkyCoord
+    >>> from sunpy.coordinates.frames import HeliographicStonyhurst
+    >>> from sunpy.coordinates.utils import get_rectangle_coordinates
+
+    >>> # With bottom left as a SkyCoord, width and height
+    >>> bottom_left = SkyCoord(0 * u.arcsec, 0 * u.arcsec, frame='heliographic_stonyhurst')
+    >>> width = 10 * u.arcsec
+    >>> height = 10 * u.arcsec
+    >>> bottom_left, top_right = get_rectangle_coordinates(bottom_left, width=width, height=height)
+
+    >>> # With bottom left of shape (2,)
+    >>> bottom_left_vector = SkyCoord([0 * u.arcsec, 10 * u.arcsec], [0 * u.arcsec, 10 * u.arcsec], frame='heliographic_stonyhurst')
+    >>> bottom_left, top_right = get_rectangle_coordinates(bottom_left_vector)
+
+    >>> # With bottom left as a BaseCoordinateFrame instance, width and height
+    >>> bottom_left =  HeliographicStonyhurst(0 * u.arcsec, 0 * u.arcsec)
+    >>> width = 10 * u.arcsec
+    >>> height = 10 * u.arcsec
+    >>> bottom_left, top_right = get_rectangle_coordinates(bottom_left, width=width, height=height)
+
+    Notes
+    -----
+    ``width`` is always treated as an increase in longitude, but ``bottom_left`` may have a higher
+    value of longitude than ``top_right`` due to the wrapping of the longitude angle.  Appropriate
+    care should be taken when using this function's output to define a range of longitudes.
+
+    ``height`` is always treated as an increase in latitude, but this function does not enforce
+    that ``bottom_left`` has a lower value of latitude than ``top_right``, in case that orientation
+    is valid for the intended use.
+    """
+    if not (hasattr(bottom_left, 'transform_to') and
+            hasattr(bottom_left, 'shape') and
+            hasattr(bottom_left, 'spherical')):
+        raise TypeError(
+            "Invalid input, bottom_left must be of type SkyCoord or BaseCoordinateFrame.")
+
+    if (top_right is not None and not ((hasattr(top_right, 'transform_to') and
+                                        hasattr(top_right, 'shape') and
+                                        hasattr(top_right, 'spherical')))):
+        raise TypeError("Invalid input, top_right must be of type SkyCoord or BaseCoordinateFrame.")
+
+    if bottom_left.shape == (2,) and any((x is not None for x in (width, height, top_right))):
+        raise ValueError("Invalid input, if bottom_left.shape == (2,) "
+                         "other parameters should not be passed.")
+
+    if all(x is not None for x in (width, height, top_right)):
+        raise ValueError("Invalid input, width, height and top_right "
+                         "parameters should not be passed simultaneously.")
+
+    if top_right is None and bottom_left.shape != (2,) and (width is None or height is None):
+        raise ValueError("Invalid input, either bottom_left and top_right "
+                         "or bottom_left and height and width should be provided.")
+
+    if width is not None:
+        if width < 0*u.deg:
+            raise ValueError("The specified width cannot be negative.")
+        if width > 360*u.deg:
+            raise ValueError("The specified width cannot be greater than 360 degrees.")
+
+    if height is not None:
+        if height < 0*u.deg:
+            raise ValueError("The specified height cannot be negative.")
+        if bottom_left.spherical.lat + height > 90*u.deg:
+            raise ValueError("The specified height exceeds the maximum latitude.")
+
+    if bottom_left.shape == (2,):
+        top_right = bottom_left[1]
+        bottom_left = bottom_left[0]
+
+    elif top_right is not None:
+        top_right = top_right.transform_to(bottom_left)
+
+    else:
+        # If bottom left is a ``SkyCoord``, top right is constructed
+        # as a SkyCoord using width and height. If bottom left is a
+        # ``frame``, the top right is typecasted to its respective
+        # frame. This is done to ensure that the output coordinates
+        # are of the same type.
+        top_right = SkyCoord(bottom_left.spherical.lon + width,
+                             bottom_left.spherical.lat + height,
+                             frame=bottom_left)
+
+        if isinstance(bottom_left, BaseCoordinateFrame):
+            top_right = top_right.frame
+
+    return bottom_left, top_right
