@@ -3,27 +3,35 @@
 # This module was developed with funding provided by
 # the Google Summer of Code (2013).
 
-from collections.abc import Hashable
-from datetime import datetime
-
-import pytest
 import os
+from datetime import datetime
+from collections.abc import Hashable
+
+import numpy as np
+import pytest
 
 import astropy.units as u
 from astropy import conf
 from astropy.utils.exceptions import AstropyUserWarning
 
-import numpy as np
-
-from sunpy.database import Database
-from sunpy.database.tables import FitsHeaderEntry, FitsKeyComment, Tag,\
-    DatabaseEntry, entries_from_query_result, entries_from_dir,\
-    entries_from_file, _create_display_table, WaveunitNotFoundError, \
-    entries_from_fido_search_result
-from sunpy.net import vso, Fido, attrs as net_attrs
 from sunpy.data.test import rootdir as testdir
-from sunpy.data.test.waveunit import waveunitdir, MQ_IMAGE
-
+from sunpy.data.test.waveunit import MQ_IMAGE, waveunitdir
+from sunpy.database import Database
+from sunpy.database.tables import (
+    DatabaseEntry,
+    FitsHeaderEntry,
+    FitsKeyComment,
+    Tag,
+    WaveunitNotFoundError,
+    _create_display_table,
+    entries_from_dir,
+    entries_from_fido_search_result,
+    entries_from_file,
+    entries_from_query_result,
+)
+from sunpy.net import Fido
+from sunpy.net import attrs as net_attrs
+from sunpy.net import vso
 
 RHESSI_IMAGE = os.path.join(testdir, 'hsi_image_20101016_191218.fits')
 EIT_195_IMAGE = os.path.join(testdir, 'EIT/efz20040301.000010_s.fits')
@@ -51,7 +59,7 @@ def fido_search_result():
         net_attrs.Instrument('XRS') | net_attrs.Instrument('noaa-indices') |
         net_attrs.Instrument('noaa-predict') |
         (net_attrs.Instrument('norh') & net_attrs.Wavelength(17 * u.GHz)) |
-        net_attrs.Instrument('rhessi') |
+        (net_attrs.Instrument('rhessi') & net_attrs.Physobs("summary_lightcurve")) |
         (net_attrs.Instrument('EVE') & net_attrs.Level(0))
     )
 
@@ -65,23 +73,23 @@ def query_result():
 @pytest.fixture
 def qr_with_none_waves():
     return vso.VSOClient().search(
-        vso.attrs.Time('20121224T120049.8', '20121224T120049.8'),
-        vso.attrs.Provider('SDAC'), vso.attrs.Instrument('VIRGO'))
+        net_attrs.Time('20121224T120049.8', '20121224T120049.8'),
+        vso.attrs.Provider('SDAC'), net_attrs.Instrument('VIRGO'))
 
 
 @pytest.fixture
 def qr_block_with_missing_physobs():
     return vso.VSOClient().search(
-        vso.attrs.Time('20130805T120000', '20130805T121000'),
-        vso.attrs.Instrument('SWAVES'), vso.attrs.Source('STEREO_A'),
-        vso.attrs.Provider('SSC'), vso.attrs.Wavelength(10 * u.kHz, 160 * u.kHz))[0]
+        net_attrs.Time('20130805T120000', '20130805T121000'),
+        net_attrs.Instrument('SWAVES'), vso.attrs.Source('STEREO_A'),
+        vso.attrs.Provider('SSC'), net_attrs.Wavelength(10 * u.kHz, 160 * u.kHz))[0]
 
 
 @pytest.fixture
 def qr_block_with_kev_unit():
     return vso.VSOClient().search(
-        vso.attrs.Time((2011, 9, 20, 1), (2011, 9, 20, 2)),
-        vso.attrs.Instrument('RHESSI'))[0]
+        net_attrs.Time((2011, 9, 20, 1), (2011, 9, 20, 2)),
+        net_attrs.Instrument('RHESSI'))[0]
 
 
 def test_fits_header_entry_equality():
@@ -113,7 +121,6 @@ def test_tag_hashability():
 
 
 @pytest.mark.remote_data
-@pytest.mark.flaky(reruns=5)
 def test_entries_from_fido_search_result(fido_search_result):
     entries = list(entries_from_fido_search_result(fido_search_result))
     # 66 entries for 8 instruments in fido_search_result
@@ -132,6 +139,7 @@ def test_entries_from_fido_search_result(fido_search_result):
         fileid='EVE_L1_esp_2012001_00',
         observation_time_start=datetime(2012, 1, 1, 0, 0),
         observation_time_end=datetime(2012, 1, 2, 0, 0),
+        size=-1.0,
         instrument='EVE',
         wavemin=0.1, wavemax=30.4)
     # 2 entries from goes
@@ -161,7 +169,7 @@ def test_entries_from_fido_search_result(fido_search_result):
     # 2 entries from norh
     assert entries[60] == DatabaseEntry(
         source='NAOJ', provider='NRO', physobs="",
-        fileid=("ftp://anonymous:data@sunpy.org@solar-pub.nao.ac.jp/"
+        fileid=("ftp://solar-pub.nao.ac.jp/"
                 "pub/nsro/norh/data/tcx/2012/01/tca120101"),
         observation_time_start=datetime(2012, 1, 1, 0, 0),
         observation_time_end=datetime(2012, 1, 2, 0, 0),
@@ -171,7 +179,7 @@ def test_entries_from_fido_search_result(fido_search_result):
     assert entries[62] == DatabaseEntry(
         source="rhessi", provider='nasa', physobs='irradiance',
         fileid=("https://hesperia.gsfc.nasa.gov/"
-                "hessidata/metadata/catalog/hsi_obssumm_20120101_016.fits"),
+                "hessidata/metadata/catalog/hsi_obssumm_20120101_032.fits"),
         observation_time_start=datetime(2012, 1, 1, 0, 0),
         observation_time_end=datetime(2012, 1, 1, 23, 59, 59, 999000),
         wavemin=np.nan, wavemax=np.nan,
@@ -204,7 +212,7 @@ def test_entries_from_fido_search_result_JSOC():
 @pytest.mark.remote_data
 def test_from_fido_search_result_block(fido_search_result):
     entry = DatabaseEntry._from_fido_search_result_block(
-        fido_search_result[0, 0][0].get_response(0)[0])
+        fido_search_result[0, 0][0].get_response(0).blocks[0])
     expected_entry = DatabaseEntry(
         source='Proba2', provider='esa', physobs='irradiance',
         fileid='http://proba2.oma.be/lyra/data/bsd/2012/01/01/lyra_20120101-000000_lev2_std.fits',
@@ -217,7 +225,7 @@ def test_from_fido_search_result_block(fido_search_result):
 
 @pytest.mark.remote_data
 def test_entry_from_qr_block(query_result):
-    entry = DatabaseEntry._from_query_result_block(query_result[0])
+    entry = DatabaseEntry._from_query_result_block(query_result.blocks[0])
     expected_entry = DatabaseEntry(
         source='SOHO', provider='SDAC', physobs='intensity',
         fileid='/archive/soho/private/data/processed/eit/lz/2001/01/efz20010101.000042',
@@ -230,7 +238,7 @@ def test_entry_from_qr_block(query_result):
 @pytest.mark.remote_data
 def test_entry_from_qr_block_with_missing_physobs(qr_block_with_missing_physobs):
     entry = DatabaseEntry._from_query_result_block(
-        qr_block_with_missing_physobs)
+        qr_block_with_missing_physobs.blocks[0])
     expected_entry = DatabaseEntry(
         source='STEREO_A', provider='SSC',
         fileid='swaves/2013/swaves_average_20130805_a_hfr.dat',
@@ -243,12 +251,15 @@ def test_entry_from_qr_block_with_missing_physobs(qr_block_with_missing_physobs)
 @pytest.mark.remote_data
 def test_entry_from_qr_block_kev(qr_block_with_kev_unit):
     # See issue #766.
-    entry = DatabaseEntry._from_query_result_block(qr_block_with_kev_unit)
+    entry = DatabaseEntry._from_query_result_block(qr_block_with_kev_unit.blocks[0])
     assert entry.source == 'RHESSI'
     assert entry.provider == 'LSSP'
-    assert entry.fileid == '/hessidata/2011/09/19/hsi_20110919_233340_002.fits'
-    assert entry.observation_time_start == datetime(2011, 9, 19, 23, 33, 40)
-    assert entry.observation_time_end == datetime(2011, 9, 20, 1, 9, 20)
+    assert entry.fileid in ['/hessidata/2011/09/19/hsi_20110919_233340_002.fits',
+                            "/hessidata/2011/09/20/hsi_20110920_010920_001.fits"]
+    assert entry.observation_time_start in [
+        datetime(2011, 9, 19, 23, 33, 40), datetime(2011, 9, 20, 1, 9, 20)]
+    assert entry.observation_time_end in [
+        datetime(2011, 9, 20, 1, 9, 20), datetime(2011, 9, 20, 2, 27, 40)]
     assert entry.instrument == 'RHESSI'
     assert round(entry.wavemin, 3) == 0.413
     assert round(entry.wavemax, 7) == 0.0000729
@@ -408,7 +419,7 @@ def test_entries_from_dir_recursively_true():
         entries = list(entries_from_dir(testdir, True,
                                         default_waveunit='angstrom',
                                         time_string_parse_format='%d/%m/%Y'))
-    assert len(entries) == 130
+    assert len(entries) == 131
 
 
 def test_entries_from_dir_recursively_false():
@@ -416,7 +427,7 @@ def test_entries_from_dir_recursively_false():
         entries = list(entries_from_dir(testdir, False,
                                         default_waveunit='angstrom',
                                         time_string_parse_format='%d/%m/%Y'))
-    assert len(entries) == 109
+    assert len(entries) == 110
 
 
 @pytest.mark.remote_data
@@ -527,21 +538,21 @@ def test_entry_from_query_results_with_none_wave_and_default_unit(
             source='SOHO', provider='SDAC', physobs='intensity',
             fileid='/archive/soho/private/data/processed/virgo/spm/VIRGO-SPM-BLUE-L2-MISSIONLONG.fits',
             observation_time_start=datetime(1996, 1, 23, 0, 0),
-            observation_time_end=datetime(2019, 5, 5, 23, 59),
+            observation_time_end=datetime(2019, 9, 30, 23, 59),
             instrument='VIRGO', size=32652.0, wavemin=None,
             wavemax=None),
         DatabaseEntry(
             source='SOHO', provider='SDAC', physobs='intensity',
             fileid='/archive/soho/private/data/processed/virgo/spm/VIRGO-SPM-GREEN-L2-MISSIONLONG.fits',
             observation_time_start=datetime(1996, 1, 23, 0, 0),
-            observation_time_end=datetime(2019, 5, 5, 23, 59),
+            observation_time_end=datetime(2019, 9, 30, 23, 59),
             instrument='VIRGO', size=32652.0, wavemin=None,
             wavemax=None),
         DatabaseEntry(
             source='SOHO', provider='SDAC', physobs='intensity',
             fileid='/archive/soho/private/data/processed/virgo/spm/VIRGO-SPM-RED-L2-MISSIONLONG.fits',
             observation_time_start=datetime(1996, 1, 23, 0, 0),
-            observation_time_end=datetime(2019, 5, 5, 23, 59),
+            observation_time_end=datetime(2019, 9, 30, 23, 59),
             instrument='VIRGO', size=32652.0, wavemin=None,
             wavemax=None),
         DatabaseEntry(
@@ -551,7 +562,7 @@ def test_entry_from_query_results_with_none_wave_and_default_unit(
             observation_time_end=datetime(2012, 9, 14, 0, 0),
             instrument='VIRGO', size=512000.0, wavemin=None,
             wavemax=None)
-        ]
+    ]
 
     for e in entries:
         assert e in expected
