@@ -8,12 +8,24 @@ from scipy.ndimage.interpolation import shift as sp_shift
 import astropy.units as u
 
 import sunpy.data.test
-from sunpy.image.coalignment import (_default_fmap_function, _lower_clip, _upper_clip, apply_shifts,
-                                     calculate_clipping, calculate_match_template_shift, clip_edges,
-                                     find_best_match_location, get_correlation_shifts,
-                                     mapsequence_coalign_by_match_template, match_template_to_layer,
-                                     parabolic_turning_point, repair_image_nonfinite)
+from sunpy.image.coalignment import (
+    _default_fmap_function,
+    _lower_clip,
+    _upper_clip,
+    apply_shifts,
+    calculate_clipping,
+    calculate_match_template_shift,
+    check_for_nonfinite_entries,
+    clip_edges,
+    find_best_match_location,
+    get_correlation_shifts,
+    mapsequence_coalign_by_match_template,
+    match_template_to_layer,
+    parabolic_turning_point,
+    repair_image_nonfinite,
+)
 from sunpy.map import Map, MapSequence
+from sunpy.util import SunpyDeprecationWarning, SunpyUserWarning
 
 
 @pytest.fixture
@@ -34,7 +46,7 @@ def aia171_test_shift():
 
 @pytest.fixture
 def aia171_test_map_layer(aia171_test_map):
-    return aia171_test_map.data
+    return aia171_test_map.data.astype('float32')  # SciPy 1.4 requires at least 16-bit floats
 
 
 @pytest.fixture
@@ -69,18 +81,54 @@ def test_repair_image_nonfinite():
             a = np.ones(9)
             a[i] = non_number
             b = a.reshape(3, 3)
-            c = repair_image_nonfinite(b)
+            with pytest.warns(SunpyDeprecationWarning):
+                c = repair_image_nonfinite(b)
+
             assert(np.isfinite(c).all())
 
 
+def test_check_for_nonfinite_entries():
+    with pytest.warns(None) as warning_list:
+        a = np.zeros((3, 3))
+        b = np.ones((3, 3))
+        check_for_nonfinite_entries(a, b)
+
+    assert len(warning_list) == 0
+
+    for i in range(0, 9):
+        for non_number in [np.nan, np.inf]:
+            a = np.ones(9)
+            a[i] = non_number
+            b = a.reshape(3, 3)
+
+            with pytest.warns(SunpyUserWarning,
+                              match='The layer image has nonfinite entries.') as warning_list:
+                check_for_nonfinite_entries(b, np.ones((3, 3)))
+
+            assert len(warning_list) == 1
+
+            with pytest.warns(SunpyUserWarning,
+                              match='The template image has nonfinite entries.') as warning_list:
+                check_for_nonfinite_entries(np.ones((3, 3)), b)
+
+            assert len(warning_list) == 1
+
+            with pytest.warns(None) as warning_list:
+                check_for_nonfinite_entries(b, b)
+
+            assert len(warning_list) == 2
+
+
 def test_match_template_to_layer(aia171_test_map_layer,
                                  aia171_test_template,
                                  aia171_test_map_layer_shape,
                                  aia171_test_template_shape):
 
     result = match_template_to_layer(aia171_test_map_layer, aia171_test_template)
-    assert_allclose(result.shape[0], aia171_test_map_layer_shape[0] - aia171_test_template_shape[0] + 1, )
-    assert_allclose(result.shape[1], aia171_test_map_layer_shape[1] - aia171_test_template_shape[1] + 1, )
+    assert_allclose(result.shape[0], aia171_test_map_layer_shape[0] -
+                    aia171_test_template_shape[0] + 1, )
+    assert_allclose(result.shape[1], aia171_test_map_layer_shape[1] -
+                    aia171_test_template_shape[1] + 1, )
     assert_allclose(np.max(result), 1.00, rtol=1e-2, atol=0)
 
 
@@ -119,7 +167,8 @@ def test_find_best_match_location(aia171_test_map_layer, aia171_test_template,
 
     result = match_template_to_layer(aia171_test_map_layer, aia171_test_template)
     match_location = u.Quantity(find_best_match_location(result))
-    assert_allclose(match_location.value, np.array(result.shape)/2. - 0.5 + aia171_test_shift, rtol=1e-3, atol=0)
+    assert_allclose(match_location.value, np.array(result.shape) /
+                    2. - 0.5 + aia171_test_shift, rtol=1e-3, atol=0)
 
 
 def test_lower_clip(aia171_test_clipping):
@@ -189,19 +238,23 @@ def test_calculate_match_template_shift(aia171_test_mc,
     nx = aia171_test_map_layer_shape[1]
 
     # Test to see if the code can recover the displacements.
-    test_displacements = calculate_match_template_shift(aia171_test_mc)
+    with pytest.warns(SunpyDeprecationWarning):
+        test_displacements = calculate_match_template_shift(aia171_test_mc)
     assert_allclose(test_displacements['x'], aia171_mc_arcsec_displacements['x'], rtol=5e-2, atol=0)
     assert_allclose(test_displacements['y'], aia171_mc_arcsec_displacements['y'], rtol=5e-2, atol=0)
 
     # Test setting the template as a ndarray
     template_ndarray = aia171_test_map_layer[ny // 4: 3 * ny // 4, nx // 4: 3 * nx // 4]
-    test_displacements = calculate_match_template_shift(aia171_test_mc, template=template_ndarray)
+    with pytest.warns(SunpyDeprecationWarning):
+        test_displacements = calculate_match_template_shift(
+            aia171_test_mc, template=template_ndarray)
     assert_allclose(test_displacements['x'], aia171_mc_arcsec_displacements['x'], rtol=5e-2, atol=0)
     assert_allclose(test_displacements['y'], aia171_mc_arcsec_displacements['y'], rtol=5e-2, atol=0)
 
     # Test setting the template as GenericMap
-    submap = aia171_test_map.submap([nx / 4, ny / 4]*u.pix, [3 * nx / 4, 3 * ny / 4]*u.pix)
-    test_displacements = calculate_match_template_shift(aia171_test_mc, template=submap)
+    submap = aia171_test_map.submap([nx / 4, ny / 4]*u.pix, top_right=[3 * nx / 4, 3 * ny / 4]*u.pix)
+    with pytest.warns(SunpyDeprecationWarning):
+        test_displacements = calculate_match_template_shift(aia171_test_mc, template=submap)
     assert_allclose(test_displacements['x'], aia171_mc_arcsec_displacements['x'], rtol=5e-2, atol=0)
     assert_allclose(test_displacements['y'], aia171_mc_arcsec_displacements['y'], rtol=5e-2, atol=0)
 
@@ -218,7 +271,8 @@ def test_mapsequence_coalign_by_match_template(aia171_test_mc,
     nx = aia171_test_map_layer_shape[1]
 
     # Get the calculated test displacements
-    test_displacements = calculate_match_template_shift(aia171_test_mc)
+    with pytest.warns(SunpyDeprecationWarning):
+        test_displacements = calculate_match_template_shift(aia171_test_mc)
 
     # Test passing in displacements
     test_mc = mapsequence_coalign_by_match_template(aia171_test_mc, shift=test_displacements)
@@ -228,33 +282,42 @@ def test_mapsequence_coalign_by_match_template(aia171_test_mc,
 
     # Test returning with no clipping.  Output layers should have the same size
     # as the original input layer.
-    test_mc = mapsequence_coalign_by_match_template(aia171_test_mc, clip=False)
+    with pytest.warns(SunpyDeprecationWarning):
+        test_mc = mapsequence_coalign_by_match_template(aia171_test_mc, clip=False)
     assert(test_mc[0].data.shape == aia171_test_map_layer_shape)
     assert(test_mc[1].data.shape == aia171_test_map_layer_shape)
 
     # Test the returned mapsequence using the default - clipping on.
     # All output layers should have the same size
     # which is smaller than the input by a known amount
-    test_mc = mapsequence_coalign_by_match_template(aia171_test_mc)
+    with pytest.warns(SunpyDeprecationWarning):
+        test_mc = mapsequence_coalign_by_match_template(aia171_test_mc)
     x_displacement_pixels = test_displacements['x'] / test_mc[0].scale[0]
     y_displacement_pixels = test_displacements['y'] / test_mc[0].scale[1]
     expected_clipping = calculate_clipping(y_displacement_pixels, x_displacement_pixels)
-    number_of_pixels_clipped = [np.sum(np.abs(expected_clipping[0])), np.sum(np.abs(expected_clipping[1]))]
+    number_of_pixels_clipped = [np.sum(np.abs(expected_clipping[0])),
+                                np.sum(np.abs(expected_clipping[1]))]
 
-    assert(test_mc[0].data.shape == (ny - number_of_pixels_clipped[0].value, nx - number_of_pixels_clipped[1].value))
-    assert(test_mc[1].data.shape == (ny - number_of_pixels_clipped[0].value, nx - number_of_pixels_clipped[1].value))
+    assert(test_mc[0].data.shape == (ny - number_of_pixels_clipped[0].value,
+                                     nx - number_of_pixels_clipped[1].value))
+    assert(test_mc[1].data.shape == (ny - number_of_pixels_clipped[0].value,
+                                     nx - number_of_pixels_clipped[1].value))
 
     # Test the returned mapsequence explicitly using clip=True.
     # All output layers should have the same size
     # which is smaller than the input by a known amount
-    test_mc = mapsequence_coalign_by_match_template(aia171_test_mc, clip=True)
+    with pytest.warns(SunpyDeprecationWarning):
+        test_mc = mapsequence_coalign_by_match_template(aia171_test_mc, clip=True)
     x_displacement_pixels = test_displacements['x'] / test_mc[0].scale[0]
     y_displacement_pixels = test_displacements['y'] / test_mc[0].scale[1]
     expected_clipping = calculate_clipping(y_displacement_pixels, x_displacement_pixels)
-    number_of_pixels_clipped = [np.sum(np.abs(expected_clipping[0])), np.sum(np.abs(expected_clipping[1]))]
+    number_of_pixels_clipped = [np.sum(np.abs(expected_clipping[0])),
+                                np.sum(np.abs(expected_clipping[1]))]
 
-    assert(test_mc[0].data.shape == (ny - number_of_pixels_clipped[0].value, nx - number_of_pixels_clipped[1].value))
-    assert(test_mc[1].data.shape == (ny - number_of_pixels_clipped[0].value, nx - number_of_pixels_clipped[1].value))
+    assert(test_mc[0].data.shape == (ny - number_of_pixels_clipped[0].value,
+                                     nx - number_of_pixels_clipped[1].value))
+    assert(test_mc[1].data.shape == (ny - number_of_pixels_clipped[0].value,
+                                     nx - number_of_pixels_clipped[1].value))
 
     # Test that the reference pixel of each map in the coaligned mapsequence is
     # correct.
@@ -295,7 +358,7 @@ def test_apply_shifts(aia171_test_map):
 
     # Test returning with clipping.  Output layers should be smaller than the
     # original layer by a known amount.
-    test_mc = apply_shifts(mc, astropy_displacements["y"], astropy_displacements["x"],  clip=True)
+    test_mc = apply_shifts(mc, astropy_displacements["y"], astropy_displacements["x"], clip=True)
     for i in range(0, len(test_mc.maps)):
         clipped = calculate_clipping(astropy_displacements["y"], astropy_displacements["x"])
         assert(test_mc[i].data.shape[0] == mc[i].data.shape[0] - np.max(clipped[0].value))
