@@ -14,7 +14,6 @@ from urllib.parse import urljoin
 import numpy as np
 import pandas
 
-from astropy.io import fits
 from astropy.time import Time
 
 from sunpy.data import cache
@@ -110,7 +109,8 @@ def remove_lytaf_events_from_timeseries(ts, artifacts=None,
     """
     # Check that input argument is of correct type
     if lytaf_path:
-        warn('laytaf_path is deprecated, has no effect and will be removed in SunPy 2.1.', SunpyDeprecationWarning)
+        warn('lytaf_path is deprecated, has no effect and will be removed in SunPy 2.1.',
+             SunpyDeprecationWarning)
     # Remove artifacts from time series
     data_columns = ts.data.columns
     time, channels, artifact_status = _remove_lytaf_events(
@@ -123,7 +123,7 @@ def remove_lytaf_events_from_timeseries(ts, artifacts=None,
     ts_new = copy.deepcopy(ts)
     ts_new.data = pandas.DataFrame(
         index=time, data={col: channels[i]
-                              for i, col in enumerate(data_columns)})
+                          for i, col in enumerate(data_columns)})
     if return_artifacts:
         return ts_new, artifact_status
     else:
@@ -131,8 +131,7 @@ def remove_lytaf_events_from_timeseries(ts, artifacts=None,
 
 
 def _remove_lytaf_events(time, channels=None, artifacts=None,
-                         return_artifacts=False, fitsfile=None,
-                         csvfile=None, filecolumns=None,
+                         return_artifacts=False, filecolumns=None,
                          lytaf_path=None, force_use_local_lytaf=False):
     """
     Removes periods of LYRA artifacts from a time series.
@@ -168,16 +167,6 @@ def _remove_lytaf_events(time, channels=None, artifacts=None,
         time and type of all artifacts removed.
         Default=False
 
-    fitsfile : `str`
-        file name (including file path and suffix, .fits) of output fits file
-        which is generated if this kwarg is not None.
-        Default=None, i.e. no fits file is output.
-
-    csvfile : `str`
-        file name (including file path and suffix, .csv) of output csv file
-        which is generated if this kwarg is not None.
-        Default=None, i.e. no csv file is output.
-
     filecolumns : `list` of strings
         Gives names of columns of any output files produced.  Although
         initially set to None above, the default is in fact
@@ -240,7 +229,8 @@ def _remove_lytaf_events(time, channels=None, artifacts=None,
     """
     # Check inputs
     if lytaf_path:
-        warn('laytaf_path is deprecated, has no effect and will be removed in SunPy 2.1.', SunpyDeprecationWarning)
+        warn('lytaf_path is deprecated, has no effect and will be removed in SunPy 2.1.',
+             SunpyDeprecationWarning)
     if channels and type(channels) is not list:
         raise TypeError("channels must be None or a list of numpy arrays "
                         "of dtype 'float64'.")
@@ -303,48 +293,7 @@ def _remove_lytaf_events(time, channels=None, artifacts=None,
                            "removed": lytaf[artifact_indices],
                            "not_removed": np.delete(lytaf, artifact_indices),
                            "not_found": artifacts_not_found}
-    # Output FITS file if fits kwarg is set
-    if fitsfile:
-        # Create time array of time strings rather than Time objects
-        # and verify filecolumns have been correctly input.  If None,
-        # generate generic filecolumns (see docstring of function called
-        # below.
-        string_time, filecolumns = _prep_columns(time, channels, filecolumns)
-        # Prepare column objects.
-        cols = [fits.Column(name=filecolumns[0], format="26A",
-                            array=string_time)]
-        if channels:
-            for i, f in enumerate(channels):
-                cols.append(fits.Column(name=filecolumns[i+1], format="D",
-                                        array=f))
-        coldefs = fits.ColDefs(cols)
-        tbhdu = fits.new_table(coldefs)
-        hdu = fits.PrimaryHDU()
-        tbhdulist = fits.HDUList([hdu, tbhdu])
-        # Write data to fits file.
-        tbhdulist.writeto(fitsfile)
-    # Output csv file if csv kwarg is set.
-    if csvfile:
-        # Create time array of time strings rather than Time objects
-        # and verify filecolumns have been correctly input.  If None,
-        # generate generic filecolumns (see docstring of function called
-        # below.
-        string_time, filecolumns = _prep_columns(time, channels, filecolumns)
-        # Open and write data to csv file.
-        with open(csvfile, 'w') as openfile:
-            csvwriter = csv.writer(openfile, delimiter=';')
-            # Write header.
-            csvwriter.writerow(filecolumns)
-            # Write data.
-            if not channels:
-                for i in range(len(time)):
-                    csvwriter.writerow(string_time[i])
-            else:
-                for i in range(len(time)):
-                    row = [string_time[i]]
-                    for f in channels:
-                        row.append(f[i])
-                    csvwriter.writerow(row)
+
     # Return values.
     if return_artifacts:
         if not channels:
@@ -425,7 +374,8 @@ def get_lytaf_events(start_time, end_time, lytaf_path=None,
     # Check inputs
     # Check lytaf path
     if lytaf_path:
-        warn('laytaf_path is deprecated, has no effect and will be removed in SunPy 2.1.', SunpyDeprecationWarning)
+        warn('lytaf_path is deprecated, has no effect and will be removed in SunPy 2.1.',
+             SunpyDeprecationWarning)
     # Parse start_time and end_time
     start_time = parse_time(start_time)
     end_time = parse_time(end_time)
@@ -506,13 +456,13 @@ def get_lytaf_events(start_time, end_time, lytaf_path=None,
             id_index = eventType_id.index(event_row[4])
             lytaf = np.append(lytaf,
                               np.array((Time(datetime.datetime.utcfromtimestamp(event_row[0]),
-                                        format='datetime'),
+                                             format='datetime'),
                                         Time(datetime.datetime.utcfromtimestamp(event_row[1]),
-                                        format='datetime'),
+                                             format='datetime'),
                                         Time(datetime.datetime.utcfromtimestamp(event_row[2]),
-                                        format='datetime'),
+                                             format='datetime'),
                                         Time(datetime.datetime.utcfromtimestamp(event_row[3]),
-                                        format='datetime'),
+                                             format='datetime'),
                                         eventType_type[id_index],
                                         eventType_definition[id_index]), dtype=lytaf.dtype))
         # Close file
@@ -562,7 +512,8 @@ def get_lytaf_event_types(lytaf_path=None, print_event_types=True):
     """
     # Set lytaf_path is not done by user
     if lytaf_path:
-        warn('laytaf_path is deprecated, has no effect and will be removed in SunPy 2.1.', SunpyDeprecationWarning)
+        warn('lytaf_path is deprecated, has no effect and will be removed in SunPy 2.1.',
+             SunpyDeprecationWarning)
     suffixes = ["lyra", "manual", "ppt", "science"]
     all_event_types = []
     # For each database file extract the event types and print them.
@@ -590,6 +541,7 @@ def get_lytaf_event_types(lytaf_path=None, print_event_types=True):
                        for event_type in event_types]
     return all_event_types
 
+
 @deprecated("1.1")
 def download_lytaf_database(lytaf_dir=''):
     """
