@@ -48,7 +48,7 @@ def read(filepath, hdus=None, memmap=None, **kwargs):
     ----------
     filepath : `str`
         The fits file to be read.
-    hdu: `int` or iterable
+    hdus: `int` or iterable
         The HDU indexes to read from the file.
 
     Returns
@@ -71,7 +71,9 @@ def read(filepath, hdus=None, memmap=None, **kwargs):
             elif isinstance(hdus, collections.Iterable):
                 hdulist = [hdulist[i] for i in hdus]
 
-        hdulist.verify('silentfix+warn')
+        hdulist = fits.hdu.HDUList(hdulist)
+        for h in hdulist:
+            h.verify('silentfix+warn')
 
         headers = get_header(hdulist)
         pairs = []
@@ -160,6 +162,9 @@ def write(fname, data, header, hdu_type=None, **kwargs):
     hdu_type: `None` or `~astropy.io.fits.CompImageHDU`
         `None` will return a normal FITS files.
         `~astropy.io.fits.CompImageHDU` will rice compress the FITS file.
+    kwargs :
+        Additional keyword arguments are given to
+        `~astropy.io.fits.HDUList.writeto`.
     """
     # Copy header so the one in memory is left alone while changing it for
     # write.
@@ -190,6 +195,8 @@ def header_to_fits(header):
     """
     Convert a header dict to a `~astropy.fits.Header`.
     """
+    # Copy the header to avoid modifying it in place
+    header = header.copy()
     # The comments need to be added to the header separately from the normal
     # kwargs. Find and deal with them:
     fits_header = fits.Header()
@@ -197,19 +204,34 @@ def header_to_fits(header):
     key_comments = header.pop('KEYCOMMENTS', False)
 
     for k, v in header.items():
-        if isinstance(v, fits.header._HeaderCommentaryCards):
-            if k.upper() == 'COMMENT':
-                comments = str(v).split('\n')
-                for com in comments:
-                    fits_header.add_comment(com)
-            elif k.upper() == 'HISTORY':
-                hists = str(v).split('\n')
-                for hist in hists:
-                    fits_header.add_history(hist)
-            elif k != '':
-                fits_header.append(fits.Card(k, str(v).split('\n')))
+        # Drop any keys that have non-ascii characters
+        if not fits.Card._ascii_text_re.match(str(v)):
+            warnings.warn(f'The meta key {k} is not valid ascii, dropping from the FITS header',
+                          SunpyUserWarning)
+            continue
+        # Drop any keys which are too long to save into FITS
+        if len(k) > 8:
+            warnings.warn(f"The meta key {k} is too long, dropping from the FITS header "
+                          "(maximum allowed key length is 8 characters).",
+                          SunpyUserWarning)
+            continue
 
+        if k.upper() in ('COMMENT', 'HV_COMMENT'):
+            comments = str(v).split('\n')
+            for com in comments:
+                fits_header.add_comment(com)
+        elif k.upper() == 'HISTORY':
+            hists = str(v).split('\n')
+            for hist in hists:
+                fits_header.add_history(hist)
+        elif isinstance(v, fits.header._HeaderCommentaryCards):
+            if k != '':
+                fits_header.append(fits.Card(k, str(v).split('\n')))
         else:
+            # For some horrific reason, we save a list to the wavelnth key in
+            # sources/rhessi.py. This is the least invasive fix for that stupidity.
+            if isinstance(v, list):
+                v = str(v)
             fits_header.append(fits.Card(k, v))
 
     if isinstance(key_comments, dict):
