@@ -1,34 +1,58 @@
-# -*- coding: utf-8 -*-
 """
 Created on Fri Jun 21 15:05:09 2013
 
 @author: stuart
 """
 import os
-import tempfile
 import pathlib
+import tempfile
 
-import pytest
 import numpy as np
+import pytest
+
 from astropy.io import fits
 from astropy.wcs import WCS
 
 import sunpy
-import sunpy.map
 import sunpy.data.test
+import sunpy.map
 
-
-filepath = sunpy.data.test.rootdir
+filepath = pathlib.Path(sunpy.data.test.rootdir)
 a_list_of_many = [os.fspath(f) for f in pathlib.Path(filepath, "EIT").glob("*")]
 a_fname = a_list_of_many[0]
 
+
 AIA_171_IMAGE = os.path.join(filepath, 'aia_171_level1.fits')
 RHESSI_IMAGE = os.path.join(filepath, 'hsi_image_20101016_191218.fits')
 
+amap = sunpy.map.Map(AIA_171_IMAGE)
+
+valid_map_inputs = [(a_fname, ),
+                    (pathlib.Path(a_fname), ),
+                    (filepath / "EIT", ),
+                    (os.fspath(filepath / "EIT"), ),
+                    (filepath / "EIT" / "*", ),
+                    (amap, ),
+                    (amap.data, amap.meta),
+                    ((amap.data, amap.meta), ),
+                    ]
+
+
+@pytest.mark.parametrize('args1', valid_map_inputs)
+@pytest.mark.parametrize('args2', valid_map_inputs)
+def test_two_map_inputs(args1, args2):
+    out = sunpy.map.Map(*args1, *args2)
+    if isinstance(out, list):
+        for m in out:
+            assert isinstance(m, sunpy.map.GenericMap)
+    else:
+        assert isinstance(out, sunpy.map.GenericMap)
 
 # ==============================================================================
 # Map Factory Tests
 # ==============================================================================
+
+
 class TestMap:
     def test_mapsequence(self):
         # Test making a MapSequence
@@ -141,17 +165,35 @@ class TestMap:
         pair_map = sunpy.map.Map(data, header)
         assert isinstance(pair_map, sunpy.map.GenericMap)
 
-    def test_errors(self):
+    def test_errors(self, tmpdir):
         # If directory doesn't exist, make sure it's listed in the error msg
         nonexist_dir = 'nonexist'
         directory = pathlib.Path(filepath, nonexist_dir)
         with pytest.raises(ValueError, match=nonexist_dir):
-            maps = sunpy.map.Map(os.fspath(directory))
+            sunpy.map.Map(os.fspath(directory))
 
         with pytest.raises(ValueError, match='Invalid input: 78'):
             # Check a random unsupported type (int) fails
             sunpy.map.Map(78)
 
+        # If one file failed to load, make sure it's raised as an expection.
+        p = tmpdir.mkdir("sub").join("hello.fits")
+        p.write("content")
+        files = [AIA_171_IMAGE, p.strpath]
+        with pytest.raises(OSError, match=(fr"Failed to read *")):
+            sunpy.map.Map(files)
+
+    @pytest.mark.parametrize('silence,error,match',
+                             [(True, RuntimeError, 'No maps loaded'),
+                              (False, sunpy.map.mapbase.MapMetaValidationError,
+                               'Image coordinate units for axis 1 not present in metadata.')])
+    def test_silence_errors(self, silence, error, match):
+        # Check that the correct errors are raised depending on silence_errors value
+        data = np.arange(0, 100).reshape(10, 10)
+        header = {}
+        with pytest.raises(error, match=match):
+            pair_map = sunpy.map.Map(data, header, silence_errors=silence)
+
     # requires dask array to run properly
     def test_dask_array(self):
         dask_array = pytest.importorskip('dask.array')
@@ -162,7 +204,7 @@ class TestMap:
 
     # requires sqlalchemy to run properly
     def test_databaseentry(self):
-        sqlalchemy = pytest.importorskip('sqlalchemy')
+        pytest.importorskip('sqlalchemy')
         sunpy_database = pytest.importorskip('sunpy.database')
         db = sunpy_database.Database(url='sqlite://', default_waveunit='angstrom')
         db.add_from_file(a_fname)
@@ -184,58 +226,34 @@ class TestMap:
         backin = sunpy.map.Map(afilename)
         assert isinstance(backin, sunpy.map.sources.EITMap)
 
-# ==============================================================================
-# Sources Tests
-# ==============================================================================
-    def test_sdo(self):
-        # Test an AIAMap
-        aia = sunpy.map.Map(AIA_171_IMAGE)
-        assert isinstance(aia, sunpy.map.sources.AIAMap)
-        # TODO: Test a HMIMap
 
-    def test_soho(self):
-        # Test EITMap, LASCOMap & MDIMap
-        eit = sunpy.map.Map(os.path.join(filepath, "EIT", "efz20040301.000010_s.fits"))
-        assert isinstance(eit, sunpy.map.sources.EITMap)
+@pytest.mark.remote_data
+def test_map_list_urls_cache():
+    """
+    Test for https://github.com/sunpy/sunpy/issues/4006
+    """
+    urls = ['http://jsoc.stanford.edu/SUM80/D136597189/S00000/image_lev1.fits',
+            'http://jsoc.stanford.edu/SUM79/D136597240/S00000/image_lev1.fits']
 
-        lasco = sunpy.map.Map(os.path.join(filepath, "lasco_c2_25299383_s.fts"))
-        assert isinstance(lasco, sunpy.map.sources.LASCOMap)
+    sunpy.map.Map(urls)
 
-        mdi_c = sunpy.map.Map(os.path.join(filepath, "mdi_fd_Ic_6h_01d.5871.0000_s.fits"))
-        assert isinstance(mdi_c, sunpy.map.sources.MDIMap)
 
-        mdi_m = sunpy.map.Map(os.path.join(filepath, "mdi_fd_M_96m_01d.5874.0005_s.fits"))
-        assert isinstance(mdi_m, sunpy.map.sources.MDIMap)
-
-    def test_stereo(self):
-        # Test EUVIMap & CORMap & HIMap
-        euvi = sunpy.map.Map(os.path.join(filepath, "euvi_20090615_000900_n4euA_s.fts"))
-        assert isinstance(euvi, sunpy.map.sources.EUVIMap)
-
-        cor = sunpy.map.Map(os.path.join(filepath, "cor1_20090615_000500_s4c1A.fts"))
-        assert isinstance(cor, sunpy.map.sources.CORMap)
-
-        hi = sunpy.map.Map(os.path.join(filepath, "hi_20110910_114721_s7h2A.fts"))
-        assert isinstance(hi, sunpy.map.sources.HIMap)
-
-    def test_rhessi(self):
-        # Test RHESSIMap
-        rhessi = sunpy.map.Map(RHESSI_IMAGE)
-        assert isinstance(rhessi, sunpy.map.sources.RHESSIMap)
-
-    def test_sot(self):
-        # Test SOTMap
-        sot = sunpy.map.Map(os.path.join(filepath, "FGMG4_20110214_030443.7.fits"))
-        assert isinstance(sot, sunpy.map.sources.SOTMap)
-
-    def test_swap(self):
-        # Test SWAPMap
-        swap = sunpy.map.Map(os.path.join(filepath, "swap_lv1_20140606_000113.fits"))
-        assert isinstance(swap, sunpy.map.sources.SWAPMap)
-
-    def test_xrt(self):
-        # Test XRTMap
-        xrt = sunpy.map.Map(os.path.join(filepath, "HinodeXRT.fits"))
-        assert isinstance(xrt, sunpy.map.sources.XRTMap)
-
-    # TODO: Test SXTMap
+# TODO: Test HMIMap, SXTMap
+@pytest.mark.parametrize('file, mapcls',
+                         [[filepath / 'EIT' / "efz20040301.000010_s.fits", sunpy.map.sources.EITMap],
+                          [filepath / "lasco_c2_25299383_s.fts", sunpy.map.sources.LASCOMap],
+                          [filepath / "mdi_fd_Ic_6h_01d.5871.0000_s.fits", sunpy.map.sources.MDIMap],
+                          [filepath / "mdi_fd_M_96m_01d.5874.0005_s.fits", sunpy.map.sources.MDIMap],
+                          [filepath / "euvi_20090615_000900_n4euA_s.fts", sunpy.map.sources.EUVIMap],
+                          [filepath / "cor1_20090615_000500_s4c1A.fts", sunpy.map.sources.CORMap],
+                          [filepath / "hi_20110910_114721_s7h2A.fts", sunpy.map.sources.HIMap],
+                          [AIA_171_IMAGE, sunpy.map.sources.AIAMap],
+                          [RHESSI_IMAGE, sunpy.map.sources.RHESSIMap],
+                          [filepath / "FGMG4_20110214_030443.7.fits", sunpy.map.sources.SOTMap],
+                          [filepath / "swap_lv1_20140606_000113.fits", sunpy.map.sources.SWAPMap],
+                          [filepath / "HinodeXRT.fits", sunpy.map.sources.XRTMap]
+                          ]
+                         )
+def test_sources(file, mapcls):
+    m = sunpy.map.Map(file)
+    assert isinstance(m, mapcls)
