@@ -1,21 +1,32 @@
 import os
-import pytest
 
 import numpy as np
+import pytest
+
 import astropy.units as u
-from astropy.coordinates import SkyCoord, BaseCoordinateFrame
+from astropy.coordinates import BaseCoordinateFrame, SkyCoord
 
-from sunpy.map.maputils import (all_pixel_indices_from_map,
-                                all_coordinates_from_map, map_edges, solar_angular_radius,
-                                contains_full_disk, is_all_off_disk,
-                                is_all_on_disk, contains_limb,
-                                coordinate_is_on_solar_disk, on_disk_bounding_coordinates)
-from sunpy.coordinates import HeliographicStonyhurst
 import sunpy.data.test
 import sunpy.map
+from sunpy.coordinates import HeliographicStonyhurst
+from sunpy.coordinates.utils import GreatArc
+from sunpy.map.maputils import (
+    all_coordinates_from_map,
+    all_pixel_indices_from_map,
+    contains_full_disk,
+    contains_limb,
+    coordinate_is_on_solar_disk,
+    is_all_off_disk,
+    is_all_on_disk,
+    map_edges,
+    on_disk_bounding_coordinates,
+    sample_at_coords,
+    solar_angular_radius,
+)
 
 testpath = sunpy.data.test.rootdir
 
+
 @pytest.fixture
 def aia171_test_map():
     return sunpy.map.Map(os.path.join(testpath, 'aia_171_level1.fits'))
@@ -23,22 +34,29 @@ def aia171_test_map():
 
 @pytest.fixture
 def all_off_disk_map(aia171_test_map):
-    return aia171_test_map.submap((1, 1)*u.pix, (11, 12)*u.pix)
+    return aia171_test_map.submap((1, 1)*u.pix, top_right=(11, 12)*u.pix)
 
 
 @pytest.fixture
 def all_on_disk_map(aia171_test_map):
-    return aia171_test_map.submap((30, 60)*u.pix, (50, 85)*u.pix)
+    return aia171_test_map.submap((30, 60)*u.pix, top_right=(50, 85)*u.pix)
 
 
 @pytest.fixture
 def straddles_limb_map(aia171_test_map):
-    return aia171_test_map.submap((64, 80)*u.pix, (120, 127)*u.pix)
+    return aia171_test_map.submap((64, 80)*u.pix, top_right=(120, 127)*u.pix)
 
 
 @pytest.fixture
 def sub_smap(aia171_test_map):
-    return aia171_test_map.submap((0, 0)*u.pix, (50, 60)*u.pix)
+    return aia171_test_map.submap((0, 0)*u.pix, top_right=(50, 60)*u.pix)
+
+
+@pytest.fixture
+def aia_test_arc(aia171_test_map):
+    start = SkyCoord(735 * u.arcsec, -471 * u.arcsec, frame=aia171_test_map.coordinate_frame)
+    end = SkyCoord(-100 * u.arcsec, 800 * u.arcsec, frame=aia171_test_map.coordinate_frame)
+    return GreatArc(start, end)
 
 
 def test_all_pixel_indices_from_map(sub_smap):
@@ -66,21 +84,21 @@ def test_all_coordinates_from_map(sub_smap):
 def test_map_edges(all_off_disk_map):
     edges = map_edges(all_off_disk_map)
     assert type(edges) is tuple
-    assert len(edges[2]) == 11
+    assert len(edges[2]) == 12
     assert np.all(edges[2][0] == [0, 0] * u.pix)
-    assert np.all(edges[2][10] == [0, 10] * u.pix)
+    assert np.all(edges[2][11] == [0, 11] * u.pix)
 
-    assert len(edges[3]) == 11
-    assert np.all(edges[3][0] == [9, 0] * u.pix)
-    assert np.all(edges[3][10] == [9, 10] * u.pix)
+    assert len(edges[3]) == 12
+    assert np.all(edges[3][0] == [10, 0] * u.pix)
+    assert np.all(edges[3][11] == [10, 11] * u.pix)
 
-    assert len(edges[1]) == 10
+    assert len(edges[1]) == 11
     assert np.all(edges[1][0] == [0, 0] * u.pix)
-    assert np.all(edges[1][9] == [9, 0] * u.pix)
+    assert np.all(edges[1][10] == [10, 0] * u.pix)
 
-    assert len(edges[0]) == 10
-    assert np.all(edges[0][0] == [0, 10] * u.pix)
-    assert np.all(edges[0][9] == [9, 10] * u.pix)
+    assert len(edges[0]) == 11
+    assert np.all(edges[0][0] == [0, 11] * u.pix)
+    assert np.all(edges[0][10] == [10, 11] * u.pix)
 
 
 def test_solar_angular_radius(aia171_test_map):
@@ -146,3 +164,14 @@ def test_on_disk_bounding_coordinates(aia171_test_map):
     np.testing.assert_almost_equal(bl.Ty.to(u.arcsec).value, -965.93063472, decimal=1)
     np.testing.assert_almost_equal(tr.Tx.to(u.arcsec).value, 964.27061417, decimal=1)
     np.testing.assert_almost_equal(tr.Ty.to(u.arcsec).value, 971.63586861, decimal=1)
+
+
+def test_data_at_coordinates(aia171_test_map, aia_test_arc):
+    data = sample_at_coords(aia171_test_map, aia_test_arc.coordinates())
+    pixels = np.asarray(np.rint(
+        aia171_test_map.world_to_pixel(aia_test_arc.coordinates())), dtype=int)
+    x = pixels[0, :]
+    y = pixels[1, :]
+    intensity_along_arc = aia171_test_map.data[y, x]
+    np.testing.assert_almost_equal(data[0], intensity_along_arc[0], decimal=1)
+    np.testing.assert_almost_equal(data[-1], intensity_along_arc[-1], decimal=1)
