@@ -1,21 +1,19 @@
-# -*- coding: utf-8 -*-
 import copy
 from pathlib import Path
 from collections import OrderedDict, namedtuple
 
 import numpy as np
-from parfive import Downloader
 
 import astropy.table
 import astropy.units as u
 
-import parfive
-
 import sunpy
 from sunpy import config
-from sunpy.net.base_client import BaseClient
-from sunpy.net.vso.attrs import Time, Wavelength, _Range
+from sunpy.net._attrs import Time, Wavelength
+from sunpy.net.attr import Range
+from sunpy.net.base_client import BaseClient, BaseQueryResponse
 from sunpy.time import TimeRange
+from sunpy.util.parfive_helpers import Downloader
 
 TIME_FORMAT = config.get("general", "time_format")
 
@@ -52,23 +50,50 @@ def iter_urls(amap, url_list, time):
         yield tmp
 
 
-class QueryResponse(list):
+class QueryResponse(BaseQueryResponse):
     """
     Container of QueryResponseBlocks
     """
 
-    def __init__(self, lst):
-        super().__init__(lst)
+    def __init__(self, lst, client=None):
+        super().__init__()
+        self._data = lst
+        self._client = client
+
+    @property
+    def blocks(self):
+        return self._data
+
+    @property
+    def client(self):
+        return self._client
+
+    @client.setter
+    def client(self, client):
+        self._client = client
+
+    def __len__(self):
+        return len(self._data)
+
+    def __getitem__(self, item):
+        # Always index so a list comes back
+        if isinstance(item, int):
+            item = slice(item, item+1)
+        return type(self)(self._data[item], client=self.client)
+
+    def __iter__(self):
+        for block in self._data:
+            yield block
 
     @classmethod
-    def create(cls, amap, lst, time=None):
+    def create(cls, amap, lst, time=None, client=None):
         if time is None:
             time = [None] * len(lst)
-        return cls(iter_urls(amap, lst, time))
+        return cls(list(iter_urls(amap, lst, time)), client=client)
 
     def time_range(self):
         """
-        Returns the time-span for which records are available
+        Returns the time-span for which records are available.
         """
         return TimeRange(min(qrblock.time.start for qrblock in self),
                          max(qrblock.time.end for qrblock in self))
@@ -84,20 +109,11 @@ class QueryResponse(list):
         s.remove(None)
         return s
 
-    def __repr__(self):
-        return repr(type(self)) + repr(self._build_table())
-
-    def __str__(self):
-        return str(self._build_table())
-
-    def _repr_html_(self):
-        return self._build_table()._repr_html_()
-
-    def _build_table(self):
+    def build_table(self):
         columns = OrderedDict((('Start Time', []), ('End Time', []),
                                ('Source', []), ('Instrument', []),
                                ('Wavelength', [])))
-        for i, qrblock in enumerate(self):
+        for qrblock in self:
             columns['Start Time'].append(
                 (qrblock.time.start).strftime(TIME_FORMAT))
             columns['End Time'].append(
@@ -150,7 +166,7 @@ class GenericClient(BaseClient):
                 self.map_['TimeRange'] = TimeRange(elem.start, elem.end)
                 self.map_['Time_start'] = elem.start
                 self.map_['Time_end'] = elem.end
-            elif isinstance(elem, _Range):
+            elif isinstance(elem, Range):
                 a_min = elem.min
                 a_max = elem.max
                 if a_min == a_max:
@@ -186,7 +202,6 @@ class GenericClient(BaseClient):
         ----------
         timerange: `sunpy.time.TimeRange`
              The timerange to extract the URLs for.
-
         \\*\\*kwargs: `dict`
              Any extra keywords to refine the search. Generated from the
              attributes passed to
@@ -220,10 +235,8 @@ class GenericClient(BaseClient):
         ----------
         qres : `~sunpy.net.dataretriever.QueryResponse`
             Results to download.
-
         filenames : list
             List of base filenames (ex - "xyz.txt")
-
         path : str
             Path to download files to
 
@@ -242,7 +255,7 @@ class GenericClient(BaseClient):
             elif '{file}' not in str(path):
                 fname = path / '{file}'
 
-            temp_dict = qres[i]._map.copy()
+            temp_dict = qres.blocks[i]._map.copy()
             temp_dict['file'] = str(filename)
             fname = fname.expanduser()
             fname = Path(str(fname).format(**temp_dict))
@@ -278,8 +291,8 @@ class GenericClient(BaseClient):
         if urls:
             times = self._get_time_for_url(urls)
             if times and times is not NotImplemented:
-                return QueryResponse.create(self.map_, urls, times)
-        return QueryResponse.create(self.map_, urls)
+                return QueryResponse.create(self.map_, urls, times, client=self)
+        return QueryResponse.create(self.map_, urls, client=self)
 
     def fetch(self, qres, path=None, overwrite=False,
               progress=True, downloader=None, wait=True):
@@ -290,39 +303,33 @@ class GenericClient(BaseClient):
         ----------
         qres : `~sunpy.net.dataretriever.QueryResponse`
             Results to download.
-
         path : `str` or `pathlib.Path`, optional
             Path to the download directory, or file template including the
             ``{file}`` string which will be replaced with the filename.
-
         overwrite : `bool` or `str`, optional
             Determine how to handle downloading if a file already exists with the
             same name. If `False` the file download will be skipped and the path
             returned to the existing file, if `True` the file will be downloaded
             and the existing file will be overwritten, if `'unique'` the filename
             will be modified to be unique.
-
         progress : `bool`, optional
             If `True` show a progress bar showing how many of the total files
             have been downloaded. If `False`, no progress bar will be shown.
-
         downloader : `parfive.Downloader`, optional
             The download manager to use.
-
         wait : `bool`, optional
            If `False` ``downloader.download()`` will not be called. Only has
            any effect if `downloader` is not `None`.
 
         Returns
         -------
-
         results: `parfive.Results`
 
         """
         if path is not None:
             path = Path(path)
 
-        urls = [qrblock.url for qrblock in qres]
+        urls = [qrblock.url for qrblock in qres.blocks]
 
         filenames = [url.split('/')[-1] for url in urls]
 
