@@ -1,25 +1,24 @@
 # Author: Rishabh Sharma <rishabh.sharma.gunner@gmail.com>
 # This module was developed under funding provided by
 # Google Summer of Code 2014
-import tarfile
 import pathlib
+import tarfile
 from collections import OrderedDict
 
-from parfive import Downloader
-
 import astropy.units as u
 from astropy.time import Time, TimeDelta
 
-from ..client import GenericClient
+from sunpy.net.dataretriever import GenericClient
+from sunpy.util.parfive_helpers import Downloader
 
 __all__ = ['NOAAIndicesClient', 'NOAAPredictClient', 'SRSClient']
 
 
 class NOAAIndicesClient(GenericClient):
     """
-    Provides access to the NOAA solar cycle indices
-    from the `ftp archive <ftp://ftp.swpc.noaa.gov/pub/weekly/>`__.
+    Provides access to the NOAA solar cycle indices.
 
+    Uses the `ftp archive <ftp://ftp.swpc.noaa.gov/pub/weekly/>`__.
     This is a fixed dataset so the result is independent of the time range.
 
     Examples
@@ -27,14 +26,13 @@ class NOAAIndicesClient(GenericClient):
 
     >>> from sunpy.net import Fido, attrs as a
     >>> results = Fido.search(a.Time("2016/1/1", "2016/1/2"),
-    ...                       a.Instrument('noaa-indices'))  #doctest: +REMOTE_DATA
-    >>> results  #doctest: +REMOTE_DATA +ELLIPSIS
+    ...                       a.Instrument.noaa_indices)  #doctest: +REMOTE_DATA
+    >>> results  #doctest: +REMOTE_DATA
     <sunpy.net.fido_factory.UnifiedResponse object at ...>
     Results from 1 Provider:
     <BLANKLINE>
     1 Results from the NOAAIndicesClient:
          Start Time           End Time      Source  Instrument  Wavelength
-           str19               str19         str4     str12        str3
     ------------------- ------------------- ------ ------------ ----------
     2016-01-01 00:00:00 2016-01-02 00:00:00   sdic noaa-indices        nan
     <BLANKLINE>
@@ -78,13 +76,19 @@ class NOAAIndicesClient(GenericClient):
                 return all(chklist)
         return False
 
+    @classmethod
+    def register_values(cls):
+        from sunpy.net import attrs
+        adict = {attrs.Instrument: [
+            ('NOAA-Indices', 'Recent Solar Indices of Observed Monthly Mean Values')]}
+        return adict
+
 
 class NOAAPredictClient(GenericClient):
     """
-    Provides access to the `NOAA SWPC <https://www.swpc.noaa.gov>`__
-    predicted sunspot Number and 10.7 cm radio flux values
-    from the `ftp archive <http://services.swpc.noaa.gov/text/>`__.
+    Provides access to the NOAA SWPC predicted sunspot Number and 10.7 cm radio flux values.
 
+    Uses this `ftp archive <http://services.swpc.noaa.gov/text/>`__.
     This is a fixed prediction so the result is independent of the time range.
 
     Examples
@@ -92,14 +96,13 @@ class NOAAPredictClient(GenericClient):
 
     >>> from sunpy.net import Fido, attrs as a
     >>> results = Fido.search(a.Time("2016/1/1", "2016/1/2"),
-    ...                       a.Instrument('noaa-predict'))  #doctest: +REMOTE_DATA
-    >>> results  #doctest: +REMOTE_DATA +ELLIPSIS
+    ...                       a.Instrument.noaa_predict)  #doctest: +REMOTE_DATA
+    >>> results  #doctest: +REMOTE_DATA
     <sunpy.net.fido_factory.UnifiedResponse object at ...>
     Results from 1 Provider:
     <BLANKLINE>
     1 Results from the NOAAPredictClient:
-     Start Time           End Time      Source  Instrument  Wavelength
-       str19               str19         str4     str12        str3
+         Start Time           End Time      Source  Instrument  Wavelength
     ------------------- ------------------- ------ ------------ ----------
     2016-01-01 00:00:00 2016-01-02 00:00:00   ises noaa-predict        nan
     <BLANKLINE>
@@ -147,25 +150,31 @@ class NOAAPredictClient(GenericClient):
                 return all(chklist)
         return False
 
+    @classmethod
+    def register_values(cls):
+        from sunpy.net import attrs
+        adict = {attrs.Instrument: [
+            ('NOAA-Predict', 'Predicted Sunspot Number And Radio Flux Values With Expected Ranges.')]}
+        return adict
+
 
 class SRSClient(GenericClient):
     """
-    Provides access to the `NOAA SWPC <https://www.swpc.noaa.gov>`__
-    solar region summary data from the `ftp archive <ftp://ftp.swpc.noaa.gov/pub/warehouse/>`__.
+    Provides access to the NOAA SWPC solar region summary data.
+
+    Uses the `ftp archive <ftp://ftp.swpc.noaa.gov/pub/warehouse/>`__.
 
     Examples
     --------
-
     >>> from sunpy.net import Fido, attrs as a
     >>> results = Fido.search(a.Time("2016/1/1", "2016/1/2"),
-    ...                       a.Instrument('SOON'))  #doctest: +REMOTE_DATA
-    >>> results  #doctest: +REMOTE_DATA +ELLIPSIS
+    ...                       a.Instrument.soon)  #doctest: +REMOTE_DATA
+    >>> results  #doctest: +REMOTE_DATA
     <sunpy.net.fido_factory.UnifiedResponse object at ...>
     Results from 1 Provider:
     <BLANKLINE>
     2 Results from the SRSClient:
-     Start Time           End Time        Source  Instrument Wavelength
-       str19               str19           str9      str4       str3
+         Start Time           End Time        Source  Instrument Wavelength
     ------------------- ------------------- --------- ---------- ----------
     2016-01-01 00:00:00 2016-01-02 00:00:00 NOAA/USAF       SOON        nan
     2016-01-01 00:00:00 2016-01-02 00:00:00 NOAA/USAF       SOON        nan
@@ -173,6 +182,7 @@ class SRSClient(GenericClient):
     <BLANKLINE>
 
     """
+
     def _get_url_for_timerange(self, timerange, **kwargs):
         result = list()
         base_url = 'ftp://ftp.swpc.noaa.gov/pub/warehouse/'
@@ -293,8 +303,18 @@ class SRSClient(GenericClient):
         boolean
             answer as to whether client can service the query
         """
+        from sunpy.net import attrs as a
+        if a.Time not in [type(at) for at in query]:
+            return False
         for x in query:
             if (x.__class__.__name__ == "Instrument" and
-                str(x.value).lower() in ["soon", "srs_table"]):
+                    str(x.value).lower() in ["soon", "srs-table"]):
                 return True
         return False
+
+    @classmethod
+    def register_values(cls):
+        from sunpy.net import attrs
+        adict = {attrs.Instrument: [("SOON", "Solar Region Summary."),
+                                    ("SRS-Table", "Solar Region Summary.")]}
+        return adict
