@@ -3,13 +3,15 @@ Access the Helio Event Catalogue
 """
 import io
 
-import zeep
 from lxml import etree
+from requests import Session
+from zeep import Client
+from zeep.transports import Transport
 
 from astropy.io.votable.table import parse_single_table
 
-from sunpy.time import parse_time
 from sunpy.net.helio import parser
+from sunpy.time import parse_time
 
 __all__ = ['HECClient']
 
@@ -67,8 +69,11 @@ class HECClient:
         if link is None:
             # The default wsdl file
             link = parser.wsdl_retriever()
-
-        self.hec_client = zeep.Client(link)
+        # Disable SSL check.
+        session = Session()
+        session.verify = False
+        transport = Transport(session=session)
+        self.hec_client = Client(link, transport=transport)
 
     def time_query(self, start_time, end_time, table=None, max_records=None):
         """
