@@ -1,24 +1,26 @@
-# -*- coding: utf-8 -*-
 import os
+import copy
+import json
 import time
 import urllib
 import warnings
-from collections.abc import Sequence
+from pathlib import Path
 
 import drms
 import numpy as np
 import pandas as pd
-import astropy.units as u
-import astropy.time
+
 import astropy.table
+import astropy.time
+import astropy.units as u
 from astropy.utils.misc import isiterable
-from parfive import Downloader, Results
 
 from sunpy import config
-from sunpy.net.base_client import BaseClient
 from sunpy.net.attr import and_
+from sunpy.net.base_client import BaseClient, BaseQueryResponse
 from sunpy.net.jsoc.attrs import walker
 from sunpy.util.exceptions import SunpyUserWarning
+from sunpy.util.parfive_helpers import Downloader, Results
 
 __all__ = ['JSOCClient', 'JSOCResponse']
 
@@ -31,24 +33,28 @@ class NotExportedError(Exception):
     pass
 
 
-class JSOCResponse(Sequence):
-    def __init__(self, table=None):
+class JSOCResponse(BaseQueryResponse):
+    def __init__(self, table=None, client=None):
         """
         table : `astropy.table.Table`
         """
+        super().__init__()
+        self.table = table or astropy.table.QTable()
+        self.query_args = getattr(table, 'query_args', None)
+        self.requests = getattr(table, 'requests', None)
+        self._client = client
 
-        self.table = table
-        self.query_args = None
-        self.requests = None
+    @property
+    def client(self):
+        return self._client
 
-    def __str__(self):
-        return str(self.table)
+    @client.setter
+    def client(self, client):
+        self._client = client
 
-    def __repr__(self):
-        return repr(self.table)
-
-    def _repr_html_(self):
-        return self.table._repr_html_()
+    @property
+    def blocks(self):
+        return list(self.table.iterrows)
 
     def __len__(self):
         if self.table is None:
@@ -57,11 +63,24 @@ class JSOCResponse(Sequence):
             return len(self.table)
 
     def __getitem__(self, item):
-        return type(self)(self.table[item])
+        if isinstance(item, int):
+            item = slice(item, item + 1)
+        ret = type(self)(self.table[item])
+        ret.query_args = self.query_args
+        ret.requests = self.requests
+        ret.client = self._client
+
+        warnings.warn("Downloading of sliced JSOC results is not supported. "
+                      "All the files present in the original response will be downloaded.",
+                      SunpyUserWarning)
+        return ret
 
     def __iter__(self):
         return (t for t in [self])
 
+    def build_table(self):
+        return self.table
+
     def append(self, table):
         if self.table is None:
             self.table = table
@@ -78,7 +97,7 @@ class JSOCResponse(Sequence):
 
 class JSOCClient(BaseClient):
     """
-    This is a Client to the JSOC Data Export service.
+    This is a client to the JSOC Data Export service.
 
     It exposes a similar API to the VSO client, although the underlying model
     is more complex. The JSOC stages data before you can download it, so a JSOC
@@ -121,7 +140,7 @@ class JSOCClient(BaseClient):
 
         The response object holds the records that your query will return:
 
-        >>> print(response)   # doctest: +ELLIPSIS  +REMOTE_DATA
+        >>> print(response)   # doctest: +REMOTE_DATA
                 T_REC          TELESCOP  INSTRUME  WAVELNTH CAR_ROT
         ----------------------- -------- ---------- -------- -------
         2014.01.01_00:00:45_TAI  SDO/HMI HMI_FRONT2   6173.0    2145
@@ -322,15 +341,15 @@ class JSOCClient(BaseClient):
 
         """
 
-        return_results = JSOCResponse()
+        return_results = JSOCResponse(client=self)
         query = and_(*query)
         blocks = []
         for block in walker.create(query):
             iargs = kwargs.copy()
             iargs.update(block)
-            blocks.append(iargs)
+            # Update blocks with deep copy of iargs because in _make_recordset we use .pop() on element from iargs
+            blocks.append(copy.deepcopy(iargs))
             return_results.append(self._lookup_records(iargs))
-
         return_results.query_args = blocks
         return return_results
 
@@ -434,7 +453,7 @@ class JSOCClient(BaseClient):
         return requests
 
     def fetch(self, jsoc_response, path=None, progress=True, overwrite=False,
-              downloader=None, wait=True, sleep=10):
+              downloader=None, wait=True, sleep=10, max_conn=4, **kwargs):
         """
         Make the request for the data in a JSOC response and wait for it to be
         staged and then download the data.
@@ -458,7 +477,7 @@ class JSOCClient(BaseClient):
             and the existing file will be overwritten, if `'unique'` the filename
             will be modified to be unique.
 
-        max_conns : `int`
+        max_conn : `int`
             Maximum number of download connections.
 
         downloader : `parfive.Downloader`, optional
@@ -481,6 +500,9 @@ class JSOCClient(BaseClient):
         # Make staging request to JSOC
         responses = self.request_data(jsoc_response)
 
+        defaults = {'max_splits': 2}
+        defaults.update(kwargs)
+
         # Make response iterable
         if not isiterable(responses):
             responses = [responses]
@@ -494,10 +516,10 @@ class JSOCClient(BaseClient):
 
         return self.get_request(responses, path=path, overwrite=overwrite,
                                 progress=progress, downloader=downloader,
-                                wait=wait)
+                                wait=wait, max_conn=max_conn, **defaults)
 
     def get_request(self, requests, path=None, overwrite=False, progress=True,
-                    downloader=None, wait=True):
+                    downloader=None, wait=True, max_conn=4, **kwargs):
         """
         Query JSOC to see if the request(s) is ready for download.
 
@@ -538,6 +560,8 @@ class JSOCClient(BaseClient):
         """
         c = drms.Client()
 
+        kwargs['max_splits'] = kwargs.get('max_splits', 2)
+
         # Convert Responses to a list if not already
         if isinstance(requests, str) or not isiterable(requests):
             requests = [requests]
@@ -557,7 +581,10 @@ class JSOCClient(BaseClient):
         if path is None:
             default_dir = config.get("downloads", "download_dir")
             path = os.path.join(default_dir, '{file}')
-        elif isinstance(path, str) and '{file}' not in path:
+        elif isinstance(path, Path):
+            path = str(path)
+
+        if isinstance(path, str) and '{file}' not in path:
             path = os.path.join(path, '{file}')
 
         paths = []
@@ -573,23 +600,35 @@ class JSOCClient(BaseClient):
                 fname = os.path.expanduser(fname)
                 paths.append(fname)
 
+        if max_conn * kwargs['max_splits'] > 10:
+            warnings.warn(("JSOC does not support more than 10 parallel connections. " +
+                           "Changing the number of parallel connections to 8."), SunpyUserWarning)
+            kwargs['max_splits'] = 2
+            max_conn = 4
+
         dl_set = True
         if not downloader:
             dl_set = False
-            downloader = Downloader(progress=progress, overwrite=overwrite)
+            downloader = Downloader(progress=progress, overwrite=overwrite, max_conn=max_conn)
+        else:
+            downloader.max_conn = max_conn
 
         urls = []
         for request in requests:
             if request.status == 0:
-                for index, data in request.data.iterrows():
-                    url_dir = request.request_url + '/'
-                    urls.append(urllib.parse.urljoin(url_dir, data['filename']))
+                if request.protocol == 'as-is':
+                    urls.extend(list(request.urls.url))
+                else:
+                    for index, data in request.data.iterrows():
+                        url_dir = request.request_url + '/'
+                        urls.append(urllib.parse.urljoin(url_dir, data['filename']))
+
         if urls:
             if progress:
                 print_message = "{0} URLs found for download. Full request totalling {1}MB"
                 print(print_message.format(len(urls), request._d['size']))
             for aurl, fname in zip(urls, paths):
-                downloader.enqueue_file(aurl, filename=fname)
+                downloader.enqueue_file(aurl, filename=fname, **kwargs)
 
         if dl_set and not wait:
             return Results()
@@ -677,9 +716,9 @@ class JSOCClient(BaseClient):
             # either through PrimeKey() attribute or Time() attribute.
             if not any(x in PKEY_LIST_TIME for x in primekey):
                 timestr = '{start}-{end}{sample}'.format(
-                        start=start_time.tai.strftime("%Y.%m.%d_%H:%M:%S_TAI"),
-                        end=end_time.tai.strftime("%Y.%m.%d_%H:%M:%S_TAI"),
-                        sample=sample)
+                    start=start_time.tai.strftime("%Y.%m.%d_%H:%M:%S_TAI"),
+                    end=end_time.tai.strftime("%Y.%m.%d_%H:%M:%S_TAI"),
+                    sample=sample)
             else:
                 error_message = "Time attribute has been passed both as a Time()"\
                                 " and PrimeKey(). Please provide any one of them"\
@@ -748,8 +787,11 @@ class JSOCClient(BaseClient):
 
         if not pkstr:
             # pkstr cannot be totally empty
-            error_message = "Atleast one PrimeKey must be passed."
-            raise ValueError(error_message)
+            #
+            # Note that whilst it is technically posisble to just search by series,
+            # this is not allowed here, because some of these would be very large
+            # searches that would make JSOC sad
+            raise ValueError("Time, Wavelength or an explicit PrimeKey must be specified.")
 
         dataset = '{series}{primekey}{segment}'.format(series=series,
                                                        primekey=pkstr,
@@ -847,7 +889,75 @@ class JSOCClient(BaseClient):
 
     @classmethod
     def _can_handle_query(cls, *query):
-        chkattr = ['Series', 'Protocol', 'Notify', 'Wavelength', 'Time',
-                   'Segment', 'Keys', 'PrimeKey', 'Sample']
+        # Import here to prevent circular imports
+        from sunpy.net import attrs as a
 
-        return all([x.__class__.__name__ in chkattr for x in query])
+        required = {a.jsoc.Series}
+        optional = {a.jsoc.Protocol, a.jsoc.Notify, a.Wavelength, a.Time,
+                    a.jsoc.Segment, a.jsoc.Keys, a.jsoc.PrimeKey, a.Sample}
+        return cls.check_attr_types_in_query(query, required, optional)
+
+    @classmethod
+    def _attrs_module(cls):
+        return 'jsoc', 'sunpy.net.jsoc.attrs'
+
+    @classmethod
+    def register_values(cls):
+        # We always use the local file for now.
+        return cls.load_jsoc_values()
+
+    @staticmethod
+    def create_parse_jsoc_values():
+        """
+        Makes a network call to the VSO API that returns what keywords they support.
+        We take this list and register all the keywords as corresponding Attrs.
+        """
+        from drms import Client
+
+        here = os.path.dirname(os.path.realpath(__file__))
+
+        c = Client()
+        # Series we are after
+        data_sources = ["hmi", "mdi", "aia"]
+
+        # Now get all the information we want.
+        series_store = []
+        segments = []
+        for series in data_sources:
+            info = c.series(rf'{series}\.')
+            for item in info:
+                data = c.info(item)
+                series_store.append((data.name, data.note))
+                if not data.segments.empty:
+                    for row in data.segments.iterrows():
+                        segments.append((row[0], row[1][-1]))
+        series_store = list(set(series_store))
+        segments = list(set(segments))
+        with open(os.path.join(here, 'data', 'attrs.json'), 'w') as attrs_file:
+            keyword_info = {}
+            keyword_info["series_store"] = series_store
+            keyword_info["segments"] = segments
+            json.dump(keyword_info, attrs_file)
+
+    @staticmethod
+    def load_jsoc_values():
+        """
+        We take this list and register all the keywords as corresponding Attrs.
+
+        Returns
+        -------
+        dict
+            The constructed Attrs dictionary ready to be passed into Attr registry.
+        """
+        from sunpy.net import attrs as a
+
+        here = os.path.dirname(os.path.realpath(__file__))
+        with open(os.path.join(here, 'data', 'attrs.json'), 'r') as attrs_file:
+            keyword_info = json.load(attrs_file)
+
+        # Create attrs out of them.
+        series_dict = {a.jsoc.Series: keyword_info["series_store"]}
+        segments_dict = {a.jsoc.Segment: keyword_info["segments"]}
+        attrs = {**series_dict, **segments_dict}
+
+        return attrs
