@@ -1,18 +1,23 @@
-# -*- coding: utf-8 -*-
 import os
 import tempfile
+from unittest import mock
+
 import pandas as pd
-import astropy.table
-import astropy.time
-import astropy.units as u
 import pytest
 from parfive import Results
 
-from sunpy.net.jsoc import JSOCClient, JSOCResponse
-import sunpy.net.jsoc.attrs as attrs
-import sunpy.net.vso.attrs as vso_attrs
+import astropy.table
+import astropy.time
+import astropy.units as u
 
-client = JSOCClient()
+import sunpy.net.attrs as a
+from sunpy.net.jsoc import JSOCClient, JSOCResponse
+from sunpy.util.exceptions import SunpyUserWarning
+
+
+@pytest.fixture
+def client():
+    return JSOCClient()
 
 
 def test_jsocresponse_double():
@@ -33,28 +38,35 @@ def test_jsocresponse_single():
 
 def test_empty_jsoc_response():
     Jresp = JSOCResponse()
-    assert Jresp.table is None
+    assert len(Jresp.table) == 0
     assert Jresp.query_args is None
     assert Jresp.requests is None
-    assert str(Jresp) == 'None'
-    assert repr(Jresp) == 'None'
     assert len(Jresp) == 0
 
 
 @pytest.mark.remote_data
-def test_query():
+def test_return_query_args(client):
+    res = client.search(a.jsoc.PrimeKey('HARPNUM', 3604),
+                        a.jsoc.Series('hmi.sharp_cea_720s'),
+                        a.jsoc.Segment('Bp') & a.jsoc.Segment('magnetogram'))
+    # Because res.query_args is list that contains dict
+    assert 'primekey' in res.query_args[0]
+
+
+@pytest.mark.remote_data
+def test_query(client):
     Jresp = client.search(
-        vso_attrs.Time('2012/1/1T00:00:00', '2012/1/1T00:01:30'),
-        attrs.Series('hmi.M_45s'), vso_attrs.Sample(90 * u.second))
+        a.Time('2012/1/1T00:00:00', '2012/1/1T00:01:30'),
+        a.jsoc.Series('hmi.M_45s'), a.Sample(90 * u.second))
     assert isinstance(Jresp, JSOCResponse)
     assert len(Jresp) == 2
 
 
 @pytest.mark.remote_data
-def test_post_pass():
+def test_post_pass(client):
     responses = client.search(
-        vso_attrs.Time('2012/1/1T00:00:00', '2012/1/1T00:00:45'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'))
+        a.Time('2012/1/1T00:00:00', '2012/1/1T00:00:45'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'))
     aa = client.request_data(responses)
     tmpresp = aa._d
     assert tmpresp['protocol'] == 'fits'
@@ -62,11 +74,23 @@ def test_post_pass():
 
 
 @pytest.mark.remote_data
-def test_post_wavelength():
+def test_build_table(client):
     responses = client.search(
-        vso_attrs.Time('2010/07/30T13:30:00', '2010/07/30T14:00:00'),
-        attrs.Series('aia.lev1_euv_12s'), attrs.Wavelength(193 * u.AA) |
-        attrs.Wavelength(335 * u.AA), attrs.Notify('jsoc@cadair.com'))
+        a.Time('2012/1/1T00:00:00', '2012/1/1T00:00:45'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'))
+    table = responses.build_table()
+    assert isinstance(table, astropy.table.Table)
+
+    columns = ['T_REC', 'TELESCOP', 'INSTRUME', 'WAVELNTH', 'CAR_ROT']
+    assert columns == table.colnames
+
+
+@pytest.mark.remote_data
+def test_post_wavelength(client):
+    responses = client.search(
+        a.Time('2010/07/30T13:30:00', '2010/07/30T14:00:00'),
+        a.jsoc.Series('aia.lev1_euv_12s'), a.jsoc.Wavelength(193 * u.AA) |
+        a.jsoc.Wavelength(335 * u.AA), a.jsoc.Notify('jsoc@cadair.com'))
     aa = client.request_data(responses)
     [r.wait() for r in aa]
     tmpresp = aa[0]._d
@@ -80,28 +104,28 @@ def test_post_wavelength():
 
 
 @pytest.mark.remote_data
-def test_post_notify_fail():
+def test_post_notify_fail(client):
     responses = client.search(
-        vso_attrs.Time('2012/1/1T00:00:00', '2012/1/1T00:00:45'),
-        attrs.Series('hmi.M_45s'))
+        a.Time('2012/1/1T00:00:00', '2012/1/1T00:00:45'),
+        a.jsoc.Series('hmi.M_45s'))
     with pytest.raises(ValueError):
         client.request_data(responses)
 
 
 @pytest.mark.remote_data()
-def test_post_wave_series():
+def test_post_wave_series(client):
     with pytest.raises(TypeError):
         client.search(
-            vso_attrs.Time('2012/1/1T00:00:00', '2012/1/1T00:00:45'),
-            attrs.Series('hmi.M_45s') | attrs.Series('aia.lev1_euv_12s'),
-            attrs.Wavelength(193 * u.AA) | attrs.Wavelength(335 * u.AA))
+            a.Time('2012/1/1T00:00:00', '2012/1/1T00:00:45'),
+            a.jsoc.Series('hmi.M_45s') | a.jsoc.Series('aia.lev1_euv_12s'),
+            a.jsoc.Wavelength(193 * u.AA) | a.jsoc.Wavelength(335 * u.AA))
 
 
 @pytest.mark.remote_data
-def test_wait_get():
+def test_wait_get(client):
     responses = client.search(
-        vso_attrs.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'))
+        a.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'))
     path = tempfile.mkdtemp()
     res = client.fetch(responses, path=path)
     assert isinstance(res, Results)
@@ -109,10 +133,10 @@ def test_wait_get():
 
 
 @pytest.mark.remote_data
-def test_get_request():
+def test_get_request(client):
     responses = client.search(
-        vso_attrs.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'))
+        a.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'))
 
     bb = client.request_data(responses)
     path = tempfile.mkdtemp()
@@ -121,13 +145,13 @@ def test_get_request():
 
 
 @pytest.mark.remote_data
-def test_invalid_query():
+def test_invalid_query(client):
     with pytest.raises(ValueError):
-        client.search(vso_attrs.Time('2012/1/1T01:00:00', '2012/1/1T01:00:45'))
+        client.search(a.Time('2012/1/1T01:00:00', '2012/1/1T01:00:45'))
 
 
 @pytest.mark.remote_data
-def test_lookup_records_errors():
+def test_lookup_records_errors(client):
     d1 = {'end_time': astropy.time.Time('2014-01-01 01:00:35'),
           'start_time': astropy.time.Time('2014-01-01 00:00:35')}
     with pytest.raises(ValueError):          # Series must be specified for a JSOC Query
@@ -163,7 +187,7 @@ def test_lookup_records_errors():
 
 
 @pytest.mark.remote_data
-def test_make_recordset_errors():
+def test_make_recordset_errors(client):
     d1 = {'series': 'aia.lev1_euv_12s'}
     with pytest.raises(ValueError):
         client._make_recordset(**d1)
@@ -172,7 +196,7 @@ def test_make_recordset_errors():
         'end_time': astropy.time.Time('2014-01-01 01:00:35', scale='tai'),
         'start_time': astropy.time.Time('2014-01-01 00:00:35', scale='tai'),
         'primekey': {'T_REC': '2014.01.01_00:00:35_TAI-2014.01.01_01:00:35_TAI'}
-        })
+    })
 
     with pytest.raises(ValueError):
         client._make_recordset(**d1)
@@ -182,14 +206,14 @@ def test_make_recordset_errors():
         'start_time': astropy.time.Time('2014-01-01 00:00:35', scale='tai'),
         'wavelength': 604*u.AA,
         'primekey': {'WAVELNTH': '604'}
-        })
+    })
 
     with pytest.raises(ValueError):
         client._make_recordset(**d1)
 
 
 @pytest.mark.remote_data
-def test_make_recordset():
+def test_make_recordset(client):
     d1 = {'series': 'aia.lev1_euv_12s',
           'end_time': astropy.time.Time('2014-01-01 01:00:35', scale='tai'),
           'start_time': astropy.time.Time('2014-01-01 00:00:35', scale='tai')
@@ -238,9 +262,9 @@ def test_make_recordset():
 
 
 @pytest.mark.remote_data
-def test_search_metadata():
-    metadata = client.search_metadata(vso_attrs.Time('2014-01-01T00:00:00', '2014-01-01T00:02:00'),
-                                      attrs.Series('aia.lev1_euv_12s'), attrs.Wavelength(304*u.AA))
+def test_search_metadata(client):
+    metadata = client.search_metadata(a.Time('2014-01-01T00:00:00', '2014-01-01T00:02:00'),
+                                      a.jsoc.Series('aia.lev1_euv_12s'), a.jsoc.Wavelength(304*u.AA))
     assert isinstance(metadata, pd.DataFrame)
     assert metadata.shape == (11, 176)
     for i in metadata.index.values:
@@ -248,38 +272,38 @@ def test_search_metadata():
 
 
 @pytest.mark.remote_data
-def test_request_data_error():
+def test_request_data_error(client):
     responses = client.search(
-        vso_attrs.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'),
-        attrs.Protocol('foo'))
+        a.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'),
+        a.jsoc.Protocol('foo'))
     with pytest.raises(TypeError):
-        req = client.request_data(responses)
+        client.request_data(responses)
 
 
 @pytest.mark.remote_data
-def test_request_data_protocol():
+def test_request_data_protocol(client):
     responses = client.search(
-        vso_attrs.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'))
+        a.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'))
     req = client.request_data(responses)
     req.wait()
     assert req._d['method'] == 'url'
     assert req._d['protocol'] == 'fits'
 
     responses = client.search(
-        vso_attrs.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'),
-        attrs.Protocol('fits'))
+        a.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'),
+        a.jsoc.Protocol('fits'))
     req = client.request_data(responses)
     req.wait()
     assert req._d['method'] == 'url'
     assert req._d['protocol'] == 'fits'
 
     responses = client.search(
-        vso_attrs.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'),
-        attrs.Protocol('as-is'))
+        a.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'),
+        a.jsoc.Protocol('as-is'))
     req = client.request_data(responses)
     req.wait()
     assert req._d['method'] == 'url_quick'
@@ -287,23 +311,75 @@ def test_request_data_protocol():
 
 
 @pytest.mark.remote_data
-def test_check_request():
+def test_check_request(client):
     responses = client.search(
-        vso_attrs.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'))
+        a.Time('2012/1/1T1:00:36', '2012/1/1T01:00:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'))
     req = client.request_data(responses)
     req.wait()
     assert req.status == 0
 
 
+@pytest.mark.flaky(reruns_delay=30)
 @pytest.mark.remote_data
-def test_results_filenames():
+def test_results_filenames(client):
     responses = client.search(
-        vso_attrs.Time('2014/1/1T1:00:36', '2014/1/1T01:01:38'),
-        attrs.Series('hmi.M_45s'), attrs.Notify('jsoc@cadair.com'))
+        a.Time('2014/1/1T1:00:36', '2014/1/1T01:01:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'))
     path = tempfile.mkdtemp()
     files = client.fetch(responses, path=path)
     assert isinstance(files, Results)
     assert len(files) == len(responses)
     for hmiurl in files:
         assert os.path.isfile(hmiurl)
+
+
+@pytest.mark.flaky(reruns_delay=30)
+@pytest.mark.remote_data
+def test_results_filenames_as_is(tmp_path, client):
+    responses = client.search(
+        a.Time('2014/1/1T1:00:36', '2014/1/1T01:01:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'),
+        a.jsoc.Protocol('as-is'))
+    assert len(responses) == 2
+    files = client.fetch(responses, path=tmp_path)
+    assert isinstance(files, Results)
+    assert len(files) == len(responses)
+    for hmiurl in files:
+        assert os.path.isfile(hmiurl)
+
+
+def test_can_handle_query_no_series(client):
+    assert not client._can_handle_query(a.Time("2020/01/02", "2020/01/03"))
+    assert not client._can_handle_query(a.Wavelength(17.1*u.nm))
+    assert client._can_handle_query(a.jsoc.Series("hmi.M_45s"))
+
+
+@pytest.mark.remote_data
+def test_max_parallel_connections(client):
+    responses = client.search(
+        a.Time('2014/1/1T1:00:36', '2014/1/1T01:01:38'),
+        a.jsoc.Series('hmi.M_45s'), a.jsoc.Notify('jsoc@cadair.com'),
+        a.jsoc.Protocol("as-is"))
+
+    path = tempfile.mkdtemp()
+
+    with mock.patch(
+        "parfive.Downloader.download",
+        new_callable=mock.MagicMock
+    ) as download:
+
+        download.side_effect = ["Mocked Downloader"]
+
+        with pytest.warns(SunpyUserWarning):
+            client.fetch(responses, path=path, max_conn=5, max_splits=5)
+
+    assert download.called
+
+
+def test_jsoc_attrs(client):
+    attrs = client.load_jsoc_values()
+    assert a.jsoc.Series in attrs.keys()
+    assert a.jsoc.Segment in attrs.keys()
+    assert len(attrs[a.jsoc.Series]) != 0
+    assert len(attrs[a.jsoc.Segment]) != 0
