@@ -3,16 +3,16 @@ Provide a set of Hypothesis Strategies for various Fido related tests.
 """
 import datetime
 
-import numpy as np
 import hypothesis.strategies as st
+import numpy as np
 from hypothesis import assume
-from hypothesis.strategies import one_of, datetimes, sampled_from
+from hypothesis.strategies import datetimes, one_of, sampled_from
 
 import astropy.time
 from astropy.time import Time
 
 from sunpy.net import attrs as a
-from sunpy.time import TimeRange
+from sunpy.time import TimeRange, parse_time
 
 TimesLeapsecond = sampled_from((Time('2015-06-30T23:59:60'),
                                 Time('2012-06-30T23:59:60')))
@@ -34,7 +34,7 @@ def TimeDelta(draw):
     Timedelta strategy that limits the maximum timedelta to being positive and
     abs max is about 10 weeks + 10 days + 10 hours + 10 minutes + a bit
     """
-    keys = st.sampled_from(['weeks', 'days', 'hours', 'minutes', 'seconds'])
+    st.sampled_from(['weeks', 'days', 'hours', 'minutes', 'seconds'])
     values = st.floats(min_value=1, max_value=10)
     time_dict = {'days': st.floats(min_value=1, max_value=8),
                  'hours': st.floats(min_value=1, max_value=12),
@@ -55,7 +55,7 @@ def offline_instruments():
     Returns a strategy for any instrument that does not need the internet to do
     a query.
     """
-    offline_instr = ['lyra', 'noaa-indices', 'noaa-predict', 'soon', 'goes']
+    offline_instr = ['noaa-indices', 'noaa-predict', 'soon']
     offline_instr = st.builds(a.Instrument, st.sampled_from(offline_instr))
 
     return st.one_of(offline_instr)
@@ -66,7 +66,7 @@ def online_instruments():
     Returns a strategy for any instrument that does need the internet to do
     a query.
     """
-    online_instr = ['eve', 'rhessi', 'norh']
+    online_instr = ['lyra', 'goes', 'eve', 'rhessi', 'norh']
     online_instr = st.builds(a.Instrument, st.sampled_from(online_instr))
 
     return online_instr
@@ -116,9 +116,8 @@ def goes_time(draw, time=Times(
 
 def range_time(min_date, max_date=Time.now()):
     time = Times(
-        min_value=datetime.datetime(1981, 1, 1, 0, 0),
-        max_value=datetime.datetime(datetime.datetime.utcnow().year, 1, 1, 0, 0),
+        min_value=parse_time(min_date).datetime,
+        max_value=parse_time(max_date).datetime
     )
-    time = time.filter(lambda x: min_date < x < max_date)
 
     return time_attr(time=time)
