@@ -1,7 +1,10 @@
 """
 This module provides fundamental solar physical constants.
 """
+import io
+
 from astropy.table import Table
+from astropy.time import Time
 
 from sunpy.sun import _constants as _con
 
@@ -9,7 +12,8 @@ __all__ = [
     'get', 'find', 'print_all', 'spectral_classification', 'au', 'mass', 'equatorial_radius',
     'volume', 'surface_area', 'average_density', 'equatorial_surface_gravity',
     'effective_temperature', 'luminosity', 'mass_conversion_rate', 'escape_velocity', 'sfu',
-    'average_angular_size'
+    'average_angular_size', 'sidereal_rotation_rate', 'first_carrington_rotation',
+    'mean_synodic_period'
 ]
 
 constants = _con.physical_constants
@@ -89,19 +93,27 @@ def print_all():
     return t
 
 
-# Add a list of constants to the docs
-_lines = [
-    'The following constants are available:\n',
-    '====================== ============== ================ =================================',
-    '         Name              Value            Unit                 Description',
-    '====================== ============== ================ =================================',
-]
-for key, const in constants.items():
-    _lines.append('{:^22} {:^14.9g} {:^16} {}'.format(
-        key, const.value, const._unit_string, const.name))
-_lines.append(_lines[1])
+def _build_docstring():
+    """Build docstring containing RST-formatted table of constants."""
+    lines = ['The following constants are available:\n']
+
+    rows = []
+    for key, const in constants.items():
+        rows.append([key, const.value, const._unit_string, const.name])
+
+    table = Table(rows=rows, names=('Name', 'Value', 'Unit', 'Description'))
+    table['Value'].info.format = '14.9g'
+
+    f = io.StringIO()
+    table.write(f, format='ascii.rst')
+    lines.append(f.getvalue())
+
+    return '\n'.join(lines)
+
+
+# Add a table of constants to the docs
 if __doc__ is not None:
-    __doc__ += '\n'.join(_lines)
+    __doc__ += _build_docstring()
 
 # Spectral class is not included in physical constants since it is not a number
 spectral_classification = 'G2V'
@@ -121,3 +133,6 @@ escape_velocity = get('escape velocity')
 sfu = get('solar flux unit')
 # Observable parameters
 average_angular_size = get('average angular size')
+sidereal_rotation_rate = get('sidereal rotation rate')
+first_carrington_rotation = Time(get('first Carrington rotation (JD TT)'), format='jd', scale='tt')
+mean_synodic_period = get('mean synodic period')
