@@ -6,7 +6,6 @@ from collections import OrderedDict
 
 import matplotlib.dates
 import numpy as np
-import matplotlib as mpl
 from matplotlib import pyplot as plt
 from pandas import DataFrame
 
@@ -54,7 +53,7 @@ class XRSTimeSeries(GenericTimeSeries):
     _source = 'xrs'
 
     @peek_show
-    def peek(self, title="GOES Xray Flux"):
+    def peek(self, title="GOES Xray Flux", **kwargs):
         """
         Plots GOES XRS light curve is the usual manner. An example is shown
         below:
@@ -70,6 +69,8 @@ class XRSTimeSeries(GenericTimeSeries):
         ----------
         title : `str`. optional
             The title of the plot. Defaults to "GOES Xray Flux".
+        **kwargs : `dict`
+            Additional plot keyword arguments that are handed to `axes.plot` functions
         """
         # Check we have a timeseries valid for plotting
         self._validate_data_for_plotting()
@@ -77,18 +78,18 @@ class XRSTimeSeries(GenericTimeSeries):
         figure = plt.figure()
         axes = plt.gca()
 
-        dates = matplotlib.dates.date2num(parse_time(self.data.index).datetime)
+        dates = matplotlib.dates.date2num(parse_time(self.to_dataframe().index).datetime)
 
-        axes.plot_date(dates, self.data['xrsa'], '-',
-                       label=r'0.5--4.0 $\AA$', color='blue', lw=2)
-        axes.plot_date(dates, self.data['xrsb'], '-',
-                       label=r'1.0--8.0 $\AA$', color='red', lw=2)
+        axes.plot_date(dates, self.to_dataframe()['xrsa'], '-',
+                       label=r'0.5--4.0 $\AA$', color='blue', lw=2, **kwargs)
+        axes.plot_date(dates, self.to_dataframe()['xrsb'], '-',
+                       label=r'1.0--8.0 $\AA$', color='red', lw=2, **kwargs)
 
         axes.set_yscale("log")
         axes.set_ylim(1e-9, 1e-2)
         axes.set_title(title)
         axes.set_ylabel('Watts m$^{-2}$')
-        axes.set_xlabel(datetime.datetime.isoformat(self.data.index[0])[0:10])
+        axes.set_xlabel(datetime.datetime.isoformat(self.to_dataframe().index[0])[0:10])
 
         ax2 = axes.twinx()
         ax2.set_yscale("log")
