@@ -3,9 +3,8 @@ This module provies NOAA Solar Cycle `~sunpy.timeseries.TimeSeries` source.
 """
 from collections import OrderedDict
 
-import numpy as np
-import matplotlib as mpl
 import matplotlib.pyplot as plt
+import numpy as np
 from pandas.io.parsers import read_csv
 
 import astropy.units as u
@@ -33,11 +32,15 @@ class NOAAIndicesTimeSeries(GenericTimeSeries):
     * Radio flux at 10.7 cm is produced by `Penticon/Ottawa <https://www.ngdc.noaa.gov/stp/solar/flux.html>`_ and the units are in sfu.
     * The Ap Geomagnetic Index is produced by the United States Air Force (USAF).
 
+    .. note::
+        See the gallery example :ref:`sphx_glr_generated_gallery_plotting_solar_cycle_example.py`
+        for how to use `~sunpy.net.Fido` to retrieve the data file.
+
     Examples
     --------
     >>> import sunpy.timeseries
-    >>> import sunpy.data.sample  # doctest: +REMOTE_DATA
-    >>> noaa = sunpy.timeseries.TimeSeries(sunpy.data.sample.NOAAINDICES_TIMESERIES, source='NOAAIndices')  # doctest: +REMOTE_DATA
+    >>> noaa_url = "ftp://ftp.swpc.noaa.gov/pub/weekly/RecentIndices.txt"
+    >>> noaa = sunpy.timeseries.TimeSeries(noaa_url, source='NOAAIndices')  # doctest: +REMOTE_DATA
     >>> noaa.peek()   # doctest: +SKIP
 
     References
@@ -52,48 +55,52 @@ class NOAAIndicesTimeSeries(GenericTimeSeries):
     _source = 'noaaindices'
 
     @peek_show
-    def peek(self, type='sunspot SWO'):
+    def peek(self, type='sunspot SWO', **kwargs):
         """
         Plots NOAA Indices as a function of time. An example is shown below.
 
         .. plot::
 
             import sunpy.timeseries
-            import sunpy.data.sample
-            noaa = sunpy.timeseries.TimeSeries(sunpy.data.sample.NOAAINDICES_TIMESERIES, source='NOAAIndices')
+            noaa_url = "ftp://ftp.swpc.noaa.gov/pub/weekly/RecentIndices.txt"
+            noaa = sunpy.timeseries.TimeSeries(noaa_url, source='NOAAIndices')
             noaa.peek()
 
         Parameters
         ----------
-        type : `str`, optional
+        type : {'sunspot SWO', 'sunspot RI', 'sunspot compare', 'radio', 'geo'}, optional
             The type of plot required. Defaults to "sunspot SWO".
+        **kwargs : `dict`
+            Additional plot keyword arguments that are handed to `axes.plot` functions
         """
         # Check we have a timeseries valid for plotting
-        self._validate_data_for_ploting()
+        self._validate_data_for_plotting()
 
         figure = plt.figure()
         axes = plt.gca()
 
         if type == 'sunspot SWO':
-            axes = self.data['sunspot SWO'].plot()
-            self.data['sunspot SWO smooth'].plot()
+            axes = self.to_dataframe()['sunspot SWO'].plot(**kwargs)
+            self.to_dataframe()['sunspot SWO smooth'].plot(**kwargs)
             axes.set_ylabel('Sunspot Number')
-        if type == 'sunspot RI':
-            axes = self.data['sunspot RI'].plot()
-            self.data['sunspot RI smooth'].plot()
+        elif type == 'sunspot RI':
+            axes = self.to_dataframe()['sunspot RI'].plot(**kwargs)
+            self.to_dataframe()['sunspot RI smooth'].plot(**kwargs)
             axes.set_ylabel('Sunspot Number')
-        if type == 'sunspot compare':
-            axes = self.data['sunspot RI'].plot()
-            self.data['sunspot SWO'].plot()
+        elif type == 'sunspot compare':
+            axes = self.to_dataframe()['sunspot RI'].plot(**kwargs)
+            self.to_dataframe()['sunspot SWO'].plot(**kwargs)
             axes.set_ylabel('Sunspot Number')
-        if type == 'radio':
-            axes = self.data['radio flux'].plot()
-            self.data['radio flux smooth'].plot()
+        elif type == 'radio':
+            axes = self.to_dataframe()['radio flux'].plot(**kwargs)
+            self.to_dataframe()['radio flux smooth'].plot(**kwargs)
             axes.set_ylabel('Radio Flux [sfu]')
-        if type == 'geo':
-            axes = self.data['geomagnetic ap'].plot()
-            self.data['geomagnetic ap smooth'].plot()
+        elif type == 'geo':
+            axes = self.to_dataframe()['geomagnetic ap'].plot(**kwargs)
+            self.to_dataframe()['geomagnetic ap smooth'].plot(**kwargs)
             axes.set_ylabel('Geomagnetic AP Index')
+        else:
+            raise ValueError(f'Got unknown plot type "{type}"')
 
         axes.set_ylim(0)
         axes.set_title('Solar Cycle Progression')
@@ -171,11 +178,15 @@ class NOAAPredictIndicesTimeSeries(GenericTimeSeries):
     * The predicted radio flux at 10.7 cm is produced by
       `Penticon/Ottawa <https://www.ngdc.noaa.gov/stp/solar/flux.html>`_ and the units are in sfu.
 
+    .. note::
+        See the gallery example :ref:`sphx_glr_generated_gallery_plotting_solar_cycle_example.py`
+        for how to use `~sunpy.net.Fido` to retrieve the data file.
+
     Examples
     --------
     >>> import sunpy.timeseries
-    >>> import sunpy.data.sample  # doctest: +REMOTE_DATA
-    >>> noaa = sunpy.timeseries.TimeSeries(sunpy.data.sample.NOAAPREDICT_TIMESERIES, source='NOAAPredictIndices')  # doctest: +REMOTE_DATA
+    >>> noaa_url = 'ftp://ftp.swpc.noaa.gov/pub/weekly/Predict.txt'
+    >>> noaa = sunpy.timeseries.TimeSeries(noaa_url, source='NOAAPredictIndices')  # doctest: +REMOTE_DATA
     >>> noaa.peek()   # doctest: +SKIP
 
     References
@@ -198,24 +209,24 @@ class NOAAPredictIndicesTimeSeries(GenericTimeSeries):
         .. plot::
 
             import sunpy.timeseries
-            import sunpy.data.sample
-            noaa = sunpy.timeseries.TimeSeries(sunpy.data.sample.NOAAPREDICT_TIMESERIES, source='NOAAPredictIndices')
+            noaa_url = 'ftp://ftp.swpc.noaa.gov/pub/weekly/Predict.txt'
+            noaa = sunpy.timeseries.TimeSeries(noaa_url, source='NOAAPredictIndices')
             noaa.peek()
 
         Parameters
         ----------
         **plot_args : `dict`
-            Any additional plot arguments that should be used when plotting.
+            Additional plot keyword arguments that are handed to `axes.plot` functions
         """
         # Check we have a timeseries valid for plotting
-        self._validate_data_for_ploting()
+        self._validate_data_for_plotting()
 
         figure = plt.figure()
         axes = plt.gca()
 
-        axes = self.data['sunspot'].plot(color='b')
-        self.data['sunspot low'].plot(linestyle='--', color='b')
-        self.data['sunspot high'].plot(linestyle='--', color='b')
+        axes = self.to_dataframe()['sunspot'].plot(color='b', **plot_args)
+        self.to_dataframe()['sunspot low'].plot(linestyle='--', color='b', **plot_args)
+        self.to_dataframe()['sunspot high'].plot(linestyle='--', color='b', **plot_args)
 
         axes.set_ylim(0)
         axes.set_title('Solar Cycle Sunspot Number Prediction')
@@ -228,7 +239,6 @@ class NOAAPredictIndicesTimeSeries(GenericTimeSeries):
 
         return figure
 
-
     @staticmethod
     def _parse_file(filepath):
         """
