@@ -48,23 +48,27 @@ class TestTimeSeries:
         ts_from_list = sunpy.timeseries.TimeSeries(a_list_of_many, source='EVE', concatenate=True)
         assert isinstance(ts_from_list, sunpy.timeseries.sources.eve.EVESpWxTimeSeries)
 
-        ts_from_folder = sunpy.timeseries.TimeSeries(os.path.join(filepath, "eve"), source='EVE', concatenate=True)
+        ts_from_folder = sunpy.timeseries.TimeSeries(
+            os.path.join(filepath, "eve"), source='EVE', concatenate=True)
         assert isinstance(ts_from_folder, sunpy.timeseries.sources.eve.EVESpWxTimeSeries)
         # text the two methods get identical dataframes
         assert ts_from_list == ts_from_folder
         # test the frames have correct headings/keys (correct concatenation axis)
-        ts_from_list.columns == sunpy.timeseries.TimeSeries(a_list_of_many[0], source='EVE', concatenate=True).columns
+        ts_from_list.columns == sunpy.timeseries.TimeSeries(
+            a_list_of_many[0], source='EVE', concatenate=True).columns
 
     def test_factory_concatenate_different_source(self):
         # Test making a TimeSeries that is the concatenation of multiple files
         ts_from_list = sunpy.timeseries.TimeSeries(a_list_of_many, source='EVE', concatenate=True)
         assert isinstance(ts_from_list, sunpy.timeseries.sources.eve.EVESpWxTimeSeries)
-        ts_from_folder = sunpy.timeseries.TimeSeries(os.path.join(filepath, "eve"), source='EVE', concatenate=True)
+        ts_from_folder = sunpy.timeseries.TimeSeries(
+            os.path.join(filepath, "eve"), source='EVE', concatenate=True)
         assert isinstance(ts_from_folder, sunpy.timeseries.sources.eve.EVESpWxTimeSeries)
         # text the two methods get identical dataframes
         assert ts_from_list == ts_from_folder
         # test the frames have correct headings/keys (correct concatenation axis)
-        ts_from_list.columns == sunpy.timeseries.TimeSeries(a_list_of_many[0], source='EVE', concatenate=True).columns
+        ts_from_list.columns == sunpy.timeseries.TimeSeries(
+            a_list_of_many[0], source='EVE', concatenate=True).columns
 
     def test_factory_generate_list_of_ts(self):
         # Test making a list TimeSeries from multiple files
@@ -75,7 +79,8 @@ class TestTimeSeries:
 
     def test_factory_generate_from_glob(self):
         # Test making a TimeSeries from a glob
-        ts_from_glob = sunpy.timeseries.TimeSeries(os.path.join(filepath, "eve", "*"), source='EVE', concatenate=True)
+        ts_from_glob = sunpy.timeseries.TimeSeries(os.path.join(
+            filepath, "eve", "*"), source='EVE', concatenate=True)
         assert isinstance(ts_from_glob, sunpy.timeseries.sources.eve.EVESpWxTimeSeries)
 
 # =============================================================================
@@ -273,6 +278,7 @@ class TestTimeSeries:
         base = parse_time(datetime.datetime.today())
         times = base - TimeDelta(np.arange(24 * 60)*u.minute)
         intensity = np.sin(np.arange(0, 12 * np.pi, ((12 * np.pi) / (24*60))))
+        tr = sunpy.time.TimeRange(times[0], times[-1])
 
         # Create the data DataFrame, header MetaDict and units OrderedDict
         data = DataFrame(intensity, index=times, columns=['intensity'])
@@ -280,13 +286,16 @@ class TestTimeSeries:
         meta_md = MetaDict({'key': 'value'})
         meta_di = {'key': 'value'}
         meta_od = OrderedDict({'key': 'value'})
+        meta_obj = sunpy.timeseries.TimeSeriesMetaData(timerange=tr, colnames=['GOES'],
+                                                       meta=MetaDict({'key': 'value'}))
 
         # Create TS using different dictionary meta types
         ts_md = sunpy.timeseries.TimeSeries(data, meta_md, units)
         ts_di = sunpy.timeseries.TimeSeries(data, meta_di, units)
         ts_od = sunpy.timeseries.TimeSeries(data, meta_od, units)
-        assert ts_md == ts_di == ts_od
-        assert ts_md.meta.metadata[0][2] == ts_di.meta.metadata[0][2] == ts_od.meta.metadata[0][2]
+        ts_obj = sunpy.timeseries.TimeSeries(data, meta_obj, units)
+        assert ts_md == ts_di == ts_od == ts_obj
+        assert ts_md.meta.metadata[0][2] == ts_di.meta.metadata[0][2] == ts_od.meta.metadata[0][2] == ts_obj.meta.metadata[0][2]
 
     def test_generic_construction_ts_list(self):
         # Generate the data and the corrisponding dates
@@ -339,11 +348,13 @@ class TestTimeSeries:
         ts_concat_1 = ts_1.concatenate(ts_2)
 
         # Concatinate during construction
-        ts_concat_2 = sunpy.timeseries.TimeSeries(data, meta, units, data2, meta2, units2, concatenate=True)
+        ts_concat_2 = sunpy.timeseries.TimeSeries(
+            data, meta, units, data2, meta2, units2, concatenate=True)
         assert isinstance(ts_concat_2, sunpy.timeseries.timeseriesbase.GenericTimeSeries)
 
         # Create TS using a tuple
-        ts_concat_3 = sunpy.timeseries.TimeSeries(((data, meta, units), (data2, meta2, units2)), concatenate=True)
+        ts_concat_3 = sunpy.timeseries.TimeSeries(
+            ((data, meta, units), (data2, meta2, units2)), concatenate=True)
         assert isinstance(ts_concat_3, sunpy.timeseries.timeseriesbase.GenericTimeSeries)
         assert ts_concat_1 == ts_concat_2 == ts_concat_3
 
@@ -351,7 +362,8 @@ class TestTimeSeries:
         # Generate the data and the corresponding dates
         base = parse_time(datetime.datetime.today())
         times = base - TimeDelta(np.arange(24 * 60)*u.minute)
-        intensity = u.Quantity(np.sin(np.arange(0, 12 * np.pi, ((12 * np.pi) / (24*60)))), u.W/u.m**2)
+        intensity = u.Quantity(
+            np.sin(np.arange(0, 12 * np.pi, ((12 * np.pi) / (24*60)))), u.W/u.m**2)
 
         # Create the units and meta objects
         units = OrderedDict([('intensity', u.W/u.m**2)])
@@ -417,13 +429,16 @@ class TestTimeSeries:
             sunpy.timeseries.TimeSeries(invalid_filepath, silence_errors=True)
 
     def test_validate_units(self):
-        valid_units = OrderedDict([('Watt Per Meter Squared', u.Unit("W / m2")), ('Meter Cubed', u.Unit("m3"))])
+        valid_units = OrderedDict(
+            [('Watt Per Meter Squared', u.Unit("W / m2")), ('Meter Cubed', u.Unit("m3"))])
         assert sunpy.timeseries.TimeSeries._validate_units(valid_units)
         # Test for not having only units for values
-        invalid_units_1 = OrderedDict([('Watt Per Meter Squared', 'string'), ('Meter Cubed', u.Unit("m3"))])
+        invalid_units_1 = OrderedDict(
+            [('Watt Per Meter Squared', 'string'), ('Meter Cubed', u.Unit("m3"))])
         assert not sunpy.timeseries.TimeSeries._validate_units(invalid_units_1)
         # Test for being a MetaDict object
-        invalid_units_2 = MetaDict(OrderedDict([('Watt Per Meter Squared', u.Unit("W / m2")), ('Meter Cubed', u.Unit("m3"))]))
+        invalid_units_2 = MetaDict(OrderedDict(
+            [('Watt Per Meter Squared', u.Unit("W / m2")), ('Meter Cubed', u.Unit("m3"))]))
         assert not sunpy.timeseries.TimeSeries._validate_units(invalid_units_2)
 
     def test_validate_meta_basic(self):
@@ -431,6 +446,9 @@ class TestTimeSeries:
         assert sunpy.timeseries.TimeSeries._validate_meta(valid_meta_1)
         valid_meta_2 = OrderedDict({'key': 'value'})
         assert sunpy.timeseries.TimeSeries._validate_meta(valid_meta_2)
+        time_range = sunpy.time.TimeRange('2020-01-01 12:00', '2020-01-02 12:00')
+        valid_meta_3 = sunpy.timeseries.TimeSeriesMetaData(time_range)
+        assert sunpy.timeseries.TimeSeries._validate_meta(valid_meta_3)
         invalid_meta = []
         assert not sunpy.timeseries.TimeSeries._validate_meta(invalid_meta)
 
