@@ -24,8 +24,12 @@ from sunpy.io.header import FileHeader
 from sunpy.timeseries.timeseriesbase import GenericTimeSeries
 from sunpy.util import expand_list
 from sunpy.util.config import get_and_create_download_dir
-from sunpy.util.datatype_factory_base import (BasicRegistrationFactory, MultipleMatchError,
-                                              NoMatchError, ValidationFunctionError)
+from sunpy.util.datatype_factory_base import (
+    BasicRegistrationFactory,
+    MultipleMatchError,
+    NoMatchError,
+    ValidationFunctionError,
+)
 from sunpy.util.metadata import MetaDict
 from sunpy.util.net import download_file
 
@@ -35,21 +39,36 @@ __all__ = ['TimeSeries', 'TimeSeriesFactory', 'NoTimeSeriesFound',
 
 class TimeSeriesFactory(BasicRegistrationFactory):
     """
-    TimeSeries(*args, **kwargs)
+    A factory for generating solar timeseries objects.
 
-    TimeSeries factory class, used to create a variety of `~sunpy.timeseries.TimeSeries` objects.
-    Valid timeseries types are specified by registering them with the factory.
+    This factory takes a variety of inputs to generate
+    `~sunpy.timeseries.GenericTimeSeries` objects.
 
     Parameters
     ----------
+    \\*inputs
+        Inputs to parse for timeseries objects. See the example section for a
+        detailed list of possible inputs.
+
     source : `str`, optional
         A string to select the observational source of the data, currently
         necessary to define how files should be read for all instruments.
+
     concatenate : `bool`, optional
         Defaults to `False`.
         If set, combine any resulting list of TimeSeries objects into a single
         TimeSeries, using successive concatenate methods.
 
+    Returns
+    -------
+    `sunpy.timeseries.GenericTimeSeries`
+        If the input results in a single timeseries object that will be returned, or if ``concatenate=True``.
+
+    `list` of `~sunpy.timeseries.GenericTimeseries`
+        If multiple inputs are parsed, they will be returned in a list, unless
+        ``concatenate=True`` is set when they will be combined into a single
+        timeseries.
+
     Examples
     --------
     >>> import sunpy.timeseries
@@ -150,6 +169,8 @@ class TimeSeriesFactory(BasicRegistrationFactory):
             return True
         elif isinstance(meta, dict):
             return True
+        elif isinstance(meta, sunpy.timeseries.TimeSeriesMetaData):
+            return True
         else:
             return False
 
@@ -287,7 +308,11 @@ class TimeSeriesFactory(BasicRegistrationFactory):
                                 # to preserve multi-line comments
                                 if isinstance(args[i+1], astropy.io.fits.header.Header):
                                     args[i+1] = MetaDict(sunpy.io.header.FileHeader(args[i+1]))
-                                meta.update(args[i+1])
+                                if isinstance(args[i+1], sunpy.timeseries.TimeSeriesMetaData):
+                                    for j in args[i+1].metas:
+                                        meta.update(j)
+                                else:
+                                    meta.update(args[i+1])
                                 i += 1  # an extra increment to account for the meta
 
                 # Add a 3-tuple for this TimeSeries.
@@ -402,7 +427,7 @@ class TimeSeriesFactory(BasicRegistrationFactory):
                 # If no specific classes have been found we can read the data
                 # if we only have one data header pair:
                 if len(pairs) == 1:
-                    already_timeseries.append(GenericTimeSeries(pairs[0].data,
+                    already_timeseries.append(GenericTimeSeries(pairs[0]._data,
                                                                 pairs[0].header))
                 else:
                     raise NoMatchError("Input read by sunpy.io can not find a "
